@@ -8,12 +8,6 @@ except ImportError:
     # Python 3
     from urllib import parse  # noqa
 
-try:
-    # Python 2.7+
-    from collections import OrderedDict as odict  # noqa
-except ImportError:
-    from .ordered_dict import OrderedDict as odict  # noqa
-
 
 PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] >= 3
@@ -65,9 +59,6 @@ def setenv(key, value):
 
 def unsetenv(key):
     """Compatibility wrapper for unsetting environment variables"""
-    try:
-        del os.environ[key]
-    except KeyError:
-        pass
+    os.environ.pop(key, None)
     if hasattr(os, 'unsetenv'):
         os.unsetenv(key)
