@@ -5,12 +5,13 @@ e.g. when python raises an IOError or OSError with errno == EINTR.
 
 """
 from __future__ import division, absolute_import, unicode_literals
-import os
 import functools
-import sys
 import itertools
+import mimetypes
+import os
 import platform
 import subprocess
+import sys
 
 from .decorators import interruptable
 from .compat import ustr
@@ -19,8 +20,8 @@ from .compat import PY3
 from .compat import WIN32
 
 # /usr/include/stdlib.h
-#define EXIT_SUCCESS    0   /* Successful exit status.  */
-#define EXIT_FAILURE    1   /* Failing exit status.  */
+# #define EXIT_SUCCESS    0   /* Successful exit status.  */
+# #define EXIT_FAILURE    1   /* Failing exit status.  */
 EXIT_SUCCESS = 0
 EXIT_FAILURE = 1
 
@@ -32,10 +33,13 @@ EXIT_USAGE = 64
 EXIT_NOINPUT = 66
 EXIT_UNAVAILABLE = 69
 
+# Default encoding
+ENCODING = 'utf-8'
+
 # Some files are not in UTF-8; some other aren't in any codification.
 # Remember that GIT doesn't care about encodings (saves binary data)
 _encoding_tests = [
-    'utf-8',
+    ENCODING,
     'iso-8859-15',
     'windows1252',
     'ascii',
@@ -43,31 +47,65 @@ _encoding_tests = [
 ]
 
 
-def decode(enc, encoding=None, errors='strict'):
+class UStr(ustr):
+    """Unicode string wrapper that remembers its encoding
+
+    UStr wraps unicode strings to provide the `encoding` attribute.
+    UStr is used when decoding strings of an unknown encoding.
+    In order to generate patches that contain the original byte sequences,
+    we must preserve the original encoding when calling decode()
+    so that it can later be used when reconstructing the original
+    byte sequences.
+
+    """
+    def __new__(cls, string, encoding):
+
+        if isinstance(string, UStr):
+            if encoding != string.encoding:
+                raise ValueError('Encoding conflict: %s vs. %s'
+                                 % (string.encoding, encoding))
+            string = ustr(string)
+
+        obj = ustr.__new__(cls, string)
+        obj.encoding = encoding
+        return obj
+
+
+def decode(value, encoding=None, errors='strict'):
     """decode(encoded_string) returns an unencoded unicode string
     """
-    if enc is None or type(enc) is ustr:
-        return enc
-
-    if encoding is None:
-        encoding_tests = _encoding_tests
+    if value is None:
+        result = None
+    elif isinstance(value, ustr):
+        result = UStr(value, ENCODING)
     else:
-        encoding_tests = itertools.chain([encoding], _encoding_tests)
+        result = None
+        if encoding is None:
+            encoding_tests = _encoding_tests
+        else:
+            encoding_tests = itertools.chain([encoding], _encoding_tests)
 
-    for encoding in encoding_tests:
-        try:
-            return enc.decode(encoding, errors)
-        except:
-            pass
-    return enc.decode('utf-8', errors='ignore')
+        for enc in encoding_tests:
+            try:
+                decoded = value.decode(enc, errors)
+                result = UStr(decoded, enc)
+                break
+            except ValueError:
+                pass
+
+        if result is None:
+            decoded = value.decode(ENCODING, errors='ignore')
+            result = UStr(decoded, ENCODING)
+
+    return result
 
 
 def encode(string, encoding=None):
     """encode(unencoded_string) returns a string encoded in utf-8
     """
-    if type(string) is not ustr:
+    if not isinstance(string, ustr):
         return string
-    return string.encode(encoding or 'utf-8', 'replace')
+    return string.encode(encoding or ENCODING, 'replace')
 
 
 def mkpath(path, encoding=None):
@@ -170,7 +208,10 @@ def start_command(cmd, cwd=None, add_env=None,
             CREATE_NO_WINDOW = 0x08000000
             extra['creationflags'] = CREATE_NO_WINDOW
 
-    return subprocess.Popen(cmd, bufsize=1, stdin=stdin, stdout=stdout,
+    # Use line buffering when in text/universal_newlines mode,
+    # otherwise use the system default buffer size.
+    bufsize = 1 if universal_newlines else -1
+    return subprocess.Popen(cmd, bufsize=bufsize, stdin=stdin, stdout=stdout,
                             stderr=stderr, cwd=cwd, env=env,
                             universal_newlines=universal_newlines, **extra)
 
@@ -196,7 +237,7 @@ def communicate(proc):
     return proc.communicate()
 
 
-def run_command(cmd, encoding=None, *args, **kwargs):
+def run_command(cmd, *args, **kwargs):
     """Run the given command to completion, and return its results.
 
     This provides a simpler interface to the subprocess module.
@@ -204,28 +245,33 @@ def run_command(cmd, encoding=None, *args, **kwargs):
     The other arguments are passed on to start_command().
 
     """
+    encoding = kwargs.pop('encoding', None)
     process = start_command(cmd, *args, **kwargs)
     (output, errors) = communicate(process)
     output = decode(output, encoding=encoding)
     errors = decode(errors, encoding=encoding)
     exit_code = process.returncode
-    return (exit_code, output or '', errors or '')
+    return (exit_code,
+            output or UStr('', ENCODING),
+            errors or UStr('', ENCODING))
 
 
 @interruptable
-def _fork_posix(args, cwd=None):
+def _fork_posix(args, cwd=None, shell=False):
     """Launch a process in the background."""
     encoded_args = [encode(arg) for arg in args]
-    return subprocess.Popen(encoded_args, cwd=cwd).pid
+    return subprocess.Popen(encoded_args, cwd=cwd, shell=shell).pid
 
 
-def _fork_win32(args, cwd=None):
+def _fork_win32(args, cwd=None, shell=False):
     """Launch a background process using crazy win32 voodoo."""
     # This is probably wrong, but it works.  Windows.. wow.
     if args[0] == 'git-dag':
         # win32 can't exec python scripts
         args = [sys.executable] + args
-    args[0] = _win32_find_exe(args[0])
+
+    if not shell:
+        args[0] = _win32_find_exe(args[0])
 
     if PY3:
         # see comment in start_command()
@@ -234,7 +280,8 @@ def _fork_win32(args, cwd=None):
         argv = [encode(arg) for arg in args]
 
     DETACHED_PROCESS = 0x00000008  # Amazing!
-    return subprocess.Popen(argv, cwd=cwd, creationflags=DETACHED_PROCESS).pid
+    return subprocess.Popen(argv, cwd=cwd, creationflags=DETACHED_PROCESS,
+                            shell=shell).pid
 
 
 def _win32_find_exe(exe):
@@ -308,26 +355,38 @@ def getenv(name, default=None):
     return decode(os.getenv(name, default))
 
 
+def guess_mimetype(filename):
+    """Robustly guess a filename's mimetype"""
+    mimetype = None
+    try:
+        mimetype = mimetypes.guess_type(filename)[0]
+    except UnicodeEncodeError:
+        mimetype = mimetypes.guess_type(encode(filename))[0]
+    except TypeError:
+        mimetype = mimetypes.guess_type(decode(filename))[0]
+    return mimetype
+
+
 def xopen(path, mode='r', encoding=None):
     return open(mkpath(path, encoding=encoding), mode)
 
 
-def stdout(msg, linesep='\n'):
+def print_stdout(msg, linesep='\n'):
     msg = msg + linesep
     if PY2:
-        msg = encode(msg, encoding='utf-8')
+        msg = encode(msg, encoding=ENCODING)
     sys.stdout.write(msg)
 
 
-def stderr(msg, linesep='\n'):
+def print_stderr(msg, linesep='\n'):
     msg = msg + linesep
     if PY2:
-        msg = encode(msg, encoding='utf-8')
+        msg = encode(msg, encoding=ENCODING)
     sys.stderr.write(msg)
 
 
 def error(msg, status=EXIT_FAILURE, linesep='\n'):
-    stderr(msg, linesep=linesep)
+    print_stderr(msg, linesep=linesep)
     sys.exit(status)
 
 
@@ -341,9 +400,47 @@ chdir = wrap(mkpath, os.chdir)
 exists = wrap(mkpath, os.path.exists)
 expanduser = wrap(encode, os.path.expanduser, decorator=decode)
 if PY2:
-    getcwd = decorate(decode, os.getcwd)
+    if hasattr(os, 'getcwdu'):
+        # pylint: disable=no-member
+        getcwd = os.getcwdu
+    else:
+        getcwd = decorate(decode, os.getcwd)
 else:
     getcwd = os.getcwd
+
+
+# NOTE: find_executable() is originally from the stdlib, but starting with
+# python3.7 the stdlib no longer bundles distutils.
+def _find_executable(executable, path=None):
+    """Tries to find 'executable' in the directories listed in 'path'.
+
+    A string listing directories separated by 'os.pathsep'; defaults to
+    os.environ['PATH'].  Returns the complete filename or None if not found.
+    """
+    if path is None:
+        path = os.environ['PATH']
+
+    paths = path.split(os.pathsep)
+    _, ext = os.path.splitext(executable)
+
+    if (sys.platform == 'win32') and (ext != '.exe'):
+        executable = executable + '.exe'
+
+    if not os.path.isfile(executable):
+        for p in paths:
+            f = os.path.join(p, executable)
+            if os.path.isfile(f):
+                # the file exists, we have a shot at spawn working
+                return f
+        return None
+
+    return executable
+
+
+if PY2:
+    find_executable = wrap(mkpath, _find_executable, decorator=decode)
+else:
+    find_executable = wrap(decode, _find_executable, decorator=decode)
 isdir = wrap(mkpath, os.path.isdir)
 isfile = wrap(mkpath, os.path.isfile)
 islink = wrap(mkpath, os.path.islink)
