@@ -1,18 +1,28 @@
 # Copyright (C) 2008-2017 David Aguilar
 # Copyright (C) 2015 Daniel Harding
-"""Provides an filesystem monitoring for Linux (via inotify) and for Windows
-(via pywin32 and the ReadDirectoryChanges function)"""
-from __future__ import division, absolute_import, unicode_literals
+"""Filesystem monitor for Linux and Windows
 
+Linux monitoring uses using inotify.
+Windows monitoring uses pywin32 and the ReadDirectoryChanges function.
+
+"""
+from __future__ import division, absolute_import, unicode_literals
 import errno
 import os
 import os.path
 import select
 from threading import Lock
 
+from qtpy import QtCore
+from qtpy.QtCore import Signal
+
 from . import utils
+from . import core
+from . import gitcmds
 from . import version
-from .decorators import memoize
+from .compat import bchr
+from .i18n import N_
+from .interaction import Interaction
 
 AVAILABLE = None
 
@@ -34,32 +44,22 @@ elif utils.is_linux():
     else:
         AVAILABLE = 'inotify'
 
-from qtpy import QtCore
-from qtpy.QtCore import Signal
-
-from . import core
-from . import gitcfg
-from . import gitcmds
-from .compat import bchr
-from .git import git
-from .i18n import N_
-from .interaction import Interaction
-
 
 class _Monitor(QtCore.QObject):
 
     files_changed = Signal()
     config_changed = Signal()
 
-    def __init__(self, thread_class):
+    def __init__(self, context, thread_class):
         QtCore.QObject.__init__(self)
+        self.context = context
         self._thread_class = thread_class
         self._thread = None
 
     def start(self):
         if self._thread_class is not None:
             assert self._thread is None
-            self._thread = self._thread_class(self)
+            self._thread = self._thread_class(self.context, self)
             self._thread.start()
 
     def stop(self):
@@ -80,11 +80,12 @@ class _BaseThread(QtCore.QThread):
     #: modifications into a single signal.
     _NOTIFICATION_DELAY = 888
 
-    def __init__(self, monitor):
+    def __init__(self, context, monitor):
         QtCore.QThread.__init__(self)
+        self.context = context
         self._monitor = monitor
         self._running = True
-        self._use_check_ignore = version.check_git('check-ignore')
+        self._use_check_ignore = version.check_git(context, 'check-ignore')
         self._force_notify = False
         self._force_config = False
         self._file_paths = set()
@@ -93,9 +94,10 @@ class _BaseThread(QtCore.QThread):
     def _pending(self):
         return self._force_notify or self._file_paths or self._force_config
 
+    # pylint: disable=no-self-use
     def refresh(self):
         """Do any housekeeping necessary in response to repository changes."""
-        pass
+        return
 
     def notify(self):
         """Notifies all observers"""
@@ -110,7 +112,7 @@ class _BaseThread(QtCore.QThread):
                                        '--non-matching', '-z', '--stdin'])
             path_list = bchr(0).join(core.encode(path)
                                      for path in self._file_paths)
-            out, err = proc.communicate(path_list)
+            out, _ = proc.communicate(path_list)
             if proc.returncode:
                 do_notify = True
             else:
@@ -146,22 +148,23 @@ if AVAILABLE == 'inotify':
 
     class _InotifyThread(_BaseThread):
         _TRIGGER_MASK = (
-                inotify.IN_ATTRIB |
-                inotify.IN_CLOSE_WRITE |
-                inotify.IN_CREATE |
-                inotify.IN_DELETE |
-                inotify.IN_MODIFY |
-                inotify.IN_MOVED_FROM |
-                inotify.IN_MOVED_TO
+            inotify.IN_ATTRIB |
+            inotify.IN_CLOSE_WRITE |
+            inotify.IN_CREATE |
+            inotify.IN_DELETE |
+            inotify.IN_MODIFY |
+            inotify.IN_MOVED_FROM |
+            inotify.IN_MOVED_TO
         )
         _ADD_MASK = (
-                _TRIGGER_MASK |
-                inotify.IN_EXCL_UNLINK |
-                inotify.IN_ONLYDIR
+            _TRIGGER_MASK |
+            inotify.IN_EXCL_UNLINK |
+            inotify.IN_ONLYDIR
         )
 
-        def __init__(self, monitor):
-            _BaseThread.__init__(self, monitor)
+        def __init__(self, context, monitor):
+            _BaseThread.__init__(self, context, monitor)
+            git = context.git
             worktree = git.worktree()
             if worktree is not None:
                 worktree = core.abspath(worktree)
@@ -201,71 +204,76 @@ if AVAILABLE == 'inotify':
 
                 self.refresh()
 
-                self._log_enabled_message()
-
-                while self._running:
-                    if self._pending:
-                        timeout = self._NOTIFICATION_DELAY
-                    else:
-                        timeout = None
-                    try:
-                        events = poll_obj.poll(timeout)
-                    except OSError as e:
-                        if e.errno == errno.EINTR:
-                            continue
-                        else:
-                            raise
-                    except select.error:
-                        continue
-                    else:
-                        if not self._running:
-                            break
-                        elif not events:
-                            self.notify()
-                        else:
-                            for fd, event in events:
-                                if fd == self._inotify_fd:
-                                    self._handle_events()
+                if self._running:
+                    self._log_enabled_message()
+                    self._process_events(poll_obj)
             finally:
-                with self._lock:
-                    if self._inotify_fd is not None:
-                        os.close(self._inotify_fd)
-                        self._inotify_fd = None
-                    if self._pipe_r is not None:
-                        os.close(self._pipe_r)
-                        self._pipe_r = None
-                        os.close(self._pipe_w)
-                        self._pipe_w = None
+                self._close_fds()
+
+        def _process_events(self, poll_obj):
+            while self._running:
+                if self._pending:
+                    timeout = self._NOTIFICATION_DELAY
+                else:
+                    timeout = None
+                try:
+                    events = poll_obj.poll(timeout)
+                # pylint: disable=duplicate-except
+                except (OSError, select.error):
+                    continue
+                else:
+                    if not self._running:
+                        break
+                    elif not events:
+                        self.notify()
+                    else:
+                        for (fd, _) in events:
+                            if fd == self._inotify_fd:
+                                self._handle_events()
+
+        def _close_fds(self):
+            with self._lock:
+                if self._inotify_fd is not None:
+                    os.close(self._inotify_fd)
+                    self._inotify_fd = None
+                if self._pipe_r is not None:
+                    os.close(self._pipe_r)
+                    self._pipe_r = None
+                    os.close(self._pipe_w)
+                    self._pipe_w = None
 
         def refresh(self):
             with self._lock:
-                if self._inotify_fd is None:
-                    return
-                try:
-                    if self._worktree is not None:
-                        tracked_dirs = set(
-                                os.path.dirname(os.path.join(self._worktree,
-                                                             path))
-                                for path in gitcmds.tracked_files())
-                        self._refresh_watches(tracked_dirs,
-                                              self._worktree_wd_to_path_map,
-                                              self._worktree_path_to_wd_map)
-                    git_dirs = set()
-                    git_dirs.add(self._git_dir)
-                    for dirpath, dirnames, filenames in core.walk(
-                            os.path.join(self._git_dir, 'refs')):
-                        git_dirs.add(dirpath)
-                    self._refresh_watches(git_dirs,
-                                          self._git_dir_wd_to_path_map,
-                                          self._git_dir_path_to_wd_map)
-                    self._git_dir_wd = \
-                            self._git_dir_path_to_wd_map.get(self._git_dir)
-                except OSError as e:
-                    if e.errno == errno.ENOSPC:
-                        self._log_out_of_wds_message()
-                        self._running = False
-                    else:
-                        raise
+                self._refresh()
+
+        def _refresh(self):
+            if self._inotify_fd is None:
+                return
+            context = self.context
+            try:
+                if self._worktree is not None:
+                    tracked_dirs = set([
+                        os.path.dirname(os.path.join(self._worktree, path))
+                        for path in gitcmds.tracked_files(context)])
+                    self._refresh_watches(tracked_dirs,
+                                          self._worktree_wd_to_path_map,
+                                          self._worktree_path_to_wd_map)
+                git_dirs = set()
+                git_dirs.add(self._git_dir)
+                for dirpath, _, _ in core.walk(
+                        os.path.join(self._git_dir, 'refs')):
+                    git_dirs.add(dirpath)
+                self._refresh_watches(git_dirs,
+                                      self._git_dir_wd_to_path_map,
+                                      self._git_dir_path_to_wd_map)
+                self._git_dir_wd = \
+                    self._git_dir_path_to_wd_map.get(self._git_dir)
+            except OSError as e:
+                if e.errno == errno.ENOSPC:
+                    self._log_out_of_wds_message()
+                    self._running = False
+                else:
+                    raise
 
         def _refresh_watches(self, paths_to_watch, wd_to_path_map,
                              path_to_wd_map):
@@ -280,9 +288,8 @@ if AVAILABLE == 'inotify':
                         # This error can occur if the target of the wd was
                         # removed on the filesystem before we call
                         # inotify.rm_watch() so ignore it.
-                        pass
-                    else:
-                        raise
+                        continue
+                    raise e
             for path in paths_to_watch - watched_paths:
                 try:
                     wd = inotify.add_watch(self._inotify_fd, core.encode(path),
@@ -296,9 +303,8 @@ if AVAILABLE == 'inotify':
                         # directory referenced by path was replaced with a file
                         # before the call to inotify.add_watch().  Therefore we
                         # simply ignore them.
-                        pass
-                    else:
-                        raise
+                        continue
+                    raise e
                 else:
                     wd_to_path_map[wd] = path
                     path_to_wd_map[path] = wd
@@ -319,16 +325,16 @@ if AVAILABLE == 'inotify':
                     self._force_notify = True
             elif wd == self._git_dir_wd:
                 name = core.decode(name)
-                if name == 'HEAD' or name == 'index':
+                if name in ('HEAD', 'index'):
                     self._force_notify = True
                 elif name == 'config':
                     self._force_config = True
             elif (wd in self._git_dir_wd_to_path_map
-                    and not core.decode(name).endswith('.lock')):
+                  and not core.decode(name).endswith('.lock')):
                 self._force_notify = True
 
         def _handle_events(self):
-            for wd, mask, cookie, name in \
+            for wd, mask, _, name in \
                     inotify.read_events(self._inotify_fd):
                 if not self._force_notify:
                     self._check_event(wd, mask, name)
@@ -353,21 +359,21 @@ if AVAILABLE == 'pywin32':
 
             try:
                 self.handle = win32file.CreateFileW(
-                        path,
-                        0x0001,  # FILE_LIST_DIRECTORY
-                        win32con.FILE_SHARE_READ | win32con.FILE_SHARE_WRITE,
-                        None,
-                        win32con.OPEN_EXISTING,
-                        win32con.FILE_FLAG_BACKUP_SEMANTICS |
-                        win32con.FILE_FLAG_OVERLAPPED,
-                        None)
+                    path,
+                    0x0001,  # FILE_LIST_DIRECTORY
+                    win32con.FILE_SHARE_READ | win32con.FILE_SHARE_WRITE,
+                    None,
+                    win32con.OPEN_EXISTING,
+                    win32con.FILE_FLAG_BACKUP_SEMANTICS |
+                    win32con.FILE_FLAG_OVERLAPPED,
+                    None)
 
                 self.buffer = win32file.AllocateReadBuffer(8192)
                 self.event = win32event.CreateEvent(None, True, False, None)
                 self.overlapped = pywintypes.OVERLAPPED()
                 self.overlapped.hEvent = self.event
                 self._start()
-            except:
+            except Exception:
                 self.close()
                 raise
 
@@ -401,8 +407,9 @@ if AVAILABLE == 'pywin32':
                   win32con.FILE_NOTIFY_CHANGE_LAST_WRITE |
                   win32con.FILE_NOTIFY_CHANGE_SECURITY)
 
-        def __init__(self, monitor):
-            _BaseThread.__init__(self, monitor)
+        def __init__(self, context, monitor):
+            _BaseThread.__init__(self, context, monitor)
+            git = context.git
             worktree = git.worktree()
             if worktree is not None:
                 worktree = self._transform_path(core.abspath(worktree))
@@ -417,15 +424,6 @@ if AVAILABLE == 'pywin32':
         def _transform_path(path):
             return path.replace('\\', '/').lower()
 
-        def _read_watch(self, watch):
-            if win32event.WaitForSingleObject(watch.event, 0) \
-                    == win32event.WAIT_TIMEOUT:
-                nbytes = 0
-            else:
-                nbytes = win32file.GetOverlappedResult(watch.handle,
-                                                       watch.overlapped, False)
-            return win32file.FILE_NOTIFY_INFORMATION(watch.buffer, nbytes)
-
         def run(self):
             try:
                 with self._stop_event_lock:
@@ -469,21 +467,20 @@ if AVAILABLE == 'pywin32':
 
         def _handle_results(self):
             if self._worktree_watch is not None:
-                for action, path in self._worktree_watch.read():
+                for _, path in self._worktree_watch.read():
                     if not self._running:
                         break
                     if self._force_notify:
                         continue
                     path = self._worktree + '/' + self._transform_path(path)
                     if (path != self._git_dir
-                        and not path.startswith(self._git_dir + '/')
-                        and not os.path.isdir(path)
-                       ):
+                            and not path.startswith(self._git_dir + '/')
+                            and not os.path.isdir(path)):
                         if self._use_check_ignore:
                             self._file_paths.add(path)
                         else:
                             self._force_notify = True
-            for action, path in self._git_dir_watch.read():
+            for _, path in self._git_dir_watch.read():
                 if not self._running:
                     break
                 if self._force_notify:
@@ -495,9 +492,8 @@ if AVAILABLE == 'pywin32':
                     self._force_config = True
                     continue
                 if (path == 'head'
-                    or path == 'index'
-                    or path.startswith('refs/')
-                   ):
+                        or path == 'index'
+                        or path.startswith('refs/')):
                     self._force_notify = True
 
         def stop(self):
@@ -508,14 +504,9 @@ if AVAILABLE == 'pywin32':
             self.wait()
 
 
-@memoize
-def current():
-    return _create_instance()
-
-
-def _create_instance():
+def create(context):
     thread_class = None
-    cfg = gitcfg.current()
+    cfg = context.cfg
     if not cfg.get('cola.inotify', default=True):
         msg = N_('File system change monitoring: disabled because'
                  ' "cola.inotify" is false.\n')
@@ -533,4 +524,4 @@ def _create_instance():
             msg = N_('File system change monitoring: disabled because libc'
                      ' does not support the inotify system calls.\n')
             Interaction.log(msg)
-    return _Monitor(thread_class)
+    return _Monitor(context, thread_class)
