@@ -1,5 +1,4 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import sys
 
 from .. import core
@@ -9,6 +8,7 @@ from .. import utils
 from ..cmd import Command
 
 
+AUTOTEMPLATE = 'cola.autoloadcommittemplate'
 BACKGROUND_EDITOR = 'cola.backgroundeditor'
 BLAME_VIEWER = 'cola.blameviewer'
 BOLD_HEADERS = 'cola.boldheaders'
@@ -38,6 +38,7 @@ TEXTWIDTH = 'cola.textwidth'
 USER_EMAIL = 'user.email'
 USER_NAME = 'user.name'
 SAFE_MODE = 'cola.safemode'
+AUTOCOMPLETE_PATHS = 'cola.autocompletepaths'
 SHOW_PATH = 'cola.showpath'
 SPELL_CHECK = 'cola.spellcheck'
 THEME = 'cola.theme'
@@ -47,6 +48,7 @@ HIDPI = 'cola.hidpi'
 class Defaults(object):
     """Read-only class for holding defaults that get overridden"""
     # These should match Git's defaults for git-defined values.
+    autotemplate = False
     blame_viewer = 'git gui blame'
     bold_headers = False
     check_conflicts = True
@@ -67,6 +69,7 @@ class Defaults(object):
     merge_verbosity = 2
     save_window_settings = True
     safe_mode = False
+    autocomplete_paths = True
     show_path = True
     sort_bookmarks = True
     spellcheck = False
