@@ -1,26 +1,27 @@
 from __future__ import division, absolute_import, unicode_literals
 
+from .. import cmds
 from .. import core
 from .. import observable
 from .. import gitcmds
 from .. import utils
 from ..i18n import N_
-from ..git import git
 from ..git import STDOUT
 from ..interaction import Interaction
-from ..models import main
 
 
 class StashModel(observable.Observable):
 
-    def __init__(self):
+    def __init__(self, context):
         observable.Observable.__init__(self)
-        self.model = model = main.model()
+        self.context = context
+        self.git = context.git
+        self.model = model = context.model
         if not model.initialized:
             model.update_status()
 
-    def stash_list(self):
-        return git.stash('list')[STDOUT].splitlines()
+    def stash_list(self, *args):
+        return self.git.stash('list', *args)[STDOUT].splitlines()
 
     def is_staged(self):
         return bool(self.model.staged)
@@ -31,37 +32,42 @@ class StashModel(observable.Observable):
 
     def stash_info(self, revids=False, names=False):
         """Parses "git stash list" and returns a list of stashes."""
-        stashes = self.stash_list()
-        revids = [s[:s.index(':')] for s in stashes]
-        names = [s.split(': ', 2)[-1] for s in stashes]
+        stashes = self.stash_list(r'--format=%gd/%aD/%s')
+        split_stashes = [s.split('/', 3) for s in stashes if s]
+        stashes = ['{0}: {1}'.format(s[0], s[2]) for s in split_stashes]
+        revids = [s[0] for s in split_stashes]
+        author_dates = [s[1] for s in split_stashes]
+        names = [s[2] for s in split_stashes]
 
-        return stashes, revids, names
+        return stashes, revids, author_dates, names
 
     def stash_diff(self, rev):
+        git = self.git
         diffstat = git.stash('show', rev)[STDOUT]
         diff = git.stash('show', '-p', '--no-ext-diff', rev)[STDOUT]
         return diffstat + '\n\n' + diff
 
 
-class CommandMixin(object):
+class ApplyStash(cmds.ContextCommand):
 
-    def is_undoable(self):
-        return False
-
-
-class ApplyStash(CommandMixin):
-
-    def __init__(self, stash_index, index):
+    def __init__(self, context, stash_index, index, pop):
+        super(ApplyStash, self).__init__(context)
         self.stash_ref = 'refs/' + stash_index
         self.index = index
+        self.pop = pop
 
     def do(self):
         ref = self.stash_ref
-        if self.index:
-            args = ['apply', '--index', ref]
+        pop = self.pop
+        if pop:
+            action = 'pop'
         else:
-            args = ['apply', ref]
-        status, out, err = git.stash(*args)
+            action = 'apply'
+        if self.index:
+            args = [action, '--index', ref]
+        else:
+            args = [action, ref]
+        status, out, err = self.git.stash(*args)
         if status == 0:
             Interaction.log_status(status, out, err)
         else:
@@ -69,14 +75,17 @@ class ApplyStash(CommandMixin):
             cmdargs = core.list2cmdline(args)
             Interaction.command_error(
                 title, 'git stash ' + cmdargs, status, out, err)
+        self.model.update_status()
 
 
-class DropStash(CommandMixin):
+class DropStash(cmds.ContextCommand):
 
-    def __init__(self, stash_index):
+    def __init__(self, context, stash_index):
+        super(DropStash, self).__init__(context)
         self.stash_ref = 'refs/' + stash_index
 
     def do(self):
+        git = self.git
         status, out, err = git.stash('drop', self.stash_ref)
         if status == 0:
             Interaction.log_status(status, out, err)
@@ -86,9 +95,10 @@ class DropStash(CommandMixin):
                 title, 'git stash drop ' + self.stash_ref, status, out, err)
 
 
-class SaveStash(CommandMixin):
+class SaveStash(cmds.ContextCommand):
 
-    def __init__(self, stash_name, keep_index):
+    def __init__(self, context, stash_name, keep_index):
+        super(SaveStash, self).__init__(context)
         self.stash_name = stash_name
         self.keep_index = keep_index
 
@@ -97,7 +107,7 @@ class SaveStash(CommandMixin):
             args = ['save', '--keep-index', self.stash_name]
         else:
             args = ['save', self.stash_name]
-        status, out, err = git.stash(*args)
+        status, out, err = self.git.stash(*args)
         if status == 0:
             Interaction.log_status(status, out, err)
         else:
@@ -106,18 +116,23 @@ class SaveStash(CommandMixin):
             Interaction.command_error(
                 title, 'git stash ' + cmdargs, status, out, err)
 
+        self.model.update_status()
 
-class StashIndex(CommandMixin):
+
+class StashIndex(cmds.ContextCommand):
     """Stash the index away"""
 
-    def __init__(self, stash_name):
+    def __init__(self, context, stash_name):
+        super(StashIndex, self).__init__(context)
         self.stash_name = stash_name
 
     def do(self):
         # Manually create a stash representing the index state
+        context = self.context
+        git = self.git
         name = self.stash_name
-        branch = gitcmds.current_branch()
-        head = gitcmds.rev_parse('HEAD')
+        branch = gitcmds.current_branch(context)
+        head = gitcmds.rev_parse(context, 'HEAD')
         message = 'On %s: %s' % (branch, name)
 
         # Get the message used for the "index" commit
@@ -166,8 +181,8 @@ class StashIndex(CommandMixin):
         # changes.  The diff from stash->HEAD is a reverse diff of the stash.
         patch = utils.tmp_filename('stash')
         with core.xopen(patch, mode='wb') as patch_fd:
-            status, out, err = git.diff_tree('refs/stash', 'HEAD', '--',
-                binary=True, _stdout=patch_fd)
+            status, out, err = git.diff_tree(
+                'refs/stash', 'HEAD', '--', binary=True, _stdout=patch_fd)
             if status != 0:
                 stash_error('diff-tree', status, out, err)
                 return
@@ -182,6 +197,8 @@ class StashIndex(CommandMixin):
         else:
             stash_error('apply', status, out, err)
 
+        self.model.update_status()
+
 
 def stash_error(cmd, status, out, err):
     title = N_('Error creating stash')
