@@ -19,9 +19,9 @@ from .. import version
 from . import defs
 
 
-def about_dialog():
+def about_dialog(context):
     """Launches the Help -> About dialog"""
-    view = AboutView(qtutils.active_window())
+    view = AboutView(context, qtutils.active_window())
     view.show()
     return view
 
@@ -61,9 +61,10 @@ class ExpandingTabWidget(QtWidgets.QTabWidget):
 class AboutView(QtWidgets.QDialog):
     """Provides the git-cola 'About' dialog"""
 
-    def __init__(self, parent=None):
+    def __init__(self, context, parent=None):
         QtWidgets.QDialog.__init__(self, parent)
 
+        self.context = context
         self.setWindowTitle(N_('About git-cola'))
         self.setWindowModality(Qt.WindowModal)
 
@@ -83,7 +84,7 @@ class AboutView(QtWidgets.QDialog):
         self.logo_text_label.setFont(font)
 
         self.text = qtutils.textbrowser(text=copyright_text())
-        self.version = qtutils.textbrowser(text=version_text())
+        self.version = qtutils.textbrowser(text=version_text(context))
         self.authors = qtutils.textbrowser(text=authors_text())
         self.translators = qtutils.textbrowser(text=translators_text())
 
@@ -118,7 +119,7 @@ def copyright_text():
     return """
 Git Cola: The highly caffeinated Git GUI
 
-Copyright (C) 2007-2016 David Aguilar and contributors
+Copyright (C) 2007-2018 David Aguilar and contributors
 
 This program is free software: you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -138,9 +139,10 @@ If not, see http://www.gnu.org/licenses/.
 """
 
 
-def version_text():
-    git_version = version.git_version()
+def version_text(context):
+    git_version = version.git_version(context)
     cola_version = version.version()
+    build_version = version.build_version()
     python_path = sys.executable
     python_version = sys.version
     qt_version = qtpy.QT_VERSION
@@ -155,9 +157,27 @@ def version_text():
 
     platform_version = platform.platform()
 
+    # Only show the build version if _build_version.py exists
+    if build_version:
+        build_version = '(%s)' % build_version
+    else:
+        build_version = ''
+
+    scope = dict(
+        cola_version=cola_version,
+        build_version=build_version,
+        git_version=git_version,
+        platform_version=platform_version,
+        pyqt_api_name=pyqt_api_name,
+        pyqt_api_version=pyqt_api_version,
+        python_path=python_path,
+        python_version=python_version,
+        qt_version=qt_version,
+        qtpy_version=qtpy_version)
+
     return N_("""
         <br>
-            Git Cola version %(cola_version)s
+            Git Cola version %(cola_version)s %(build_version)s
         <ul>
             <li> %(platform_version)s
             <li> Python (%(python_path)s) %(python_version)s
@@ -166,7 +186,7 @@ def version_text():
             <li> QtPy %(qtpy_version)s
             <li> %(pyqt_api_name)s %(pyqt_api_version)s
         </ul>
-    """) % locals()
+    """) % scope
 
 
 def link(url, text, palette=None):
@@ -175,13 +195,15 @@ def link(url, text, palette=None):
 
     color = palette.color(QtGui.QPalette.Foreground)
     rgb = 'rgb(%s, %s, %s)' % (color.red(), color.green(), color.blue())
+    scope = dict(rgb=rgb, text=text, url=url)
 
-    return ("""
+    return """
         <a style="font-style: italic; text-decoration: none; color: %(rgb)s;"
             href="%(url)s">
             %(text)s
         </a>
-    """ % locals())
+    """ % scope
+
 
 def mailto(email, text, palette):
     return link('mailto:%s' % email, text, palette) + '<br>'
@@ -205,55 +227,81 @@ def render_authors(authors):
 
 def contributors_text(authors, prelude='', epilogue=''):
     author_text = render_authors(authors)
+    scope = dict(author_text=author_text, epilogue=epilogue, prelude=prelude)
 
-    return '''
+    return """
         %(prelude)s
         %(author_text)s
         %(epilogue)s
-    ''' % locals()
+    """ % scope
 
 
 def authors_text():
     palette = QtGui.QPalette()
     contact = N_('Email contributor')
     authors = (
-        dict(name='David Aguilar',
-             title=N_('Maintainer (since 2007) and developer'),
-             email=mailto('davvid@gmail.com', contact, palette)),
         # The names listed here are listed in the same order as
         # `git shortlog --summary --numbered --no-merges`
         # Please submit a pull request if you would like to include your
         # email address in the about screen.
         # See the `generate-about` script in the "todo" branch.
         # vim :read! ./Meta/generate-about
+        dict(name='David Aguilar',
+             title=N_('Maintainer (since 2007) and developer'),
+             email=mailto('davvid@gmail.com', contact, palette)),
         dict(name='Daniel Harding', title=N_('Developer')),
         dict(name='Ｖ字龍(Vdragon)', title=N_('Developer'),
              email=mailto('Vdragon.Taiwan@gmail.com', contact, palette)),
+        dict(name='Efimov Vasily', title=N_('Developer')),
         dict(name='Guillaume de Bure', title=N_('Developer')),
-        dict(name='Alex Chernetz', title=N_('Developer')),
+        dict(name='Javier Rodriguez Cuevas', title=N_('Developer')),
         dict(name='Uri Okrent', title=N_('Developer')),
-        dict(name='Thomas Kluyver', title=N_('Developer')),
+        dict(name='Alex Chernetz', title=N_('Developer')),
+        dict(name='xhl', title=N_('Developer')),
         dict(name='Andreas Sommer', title=N_('Developer')),
+        dict(name='Thomas Kluyver', title=N_('Developer')),
         dict(name='Minarto Margoliono', title=N_('Developer')),
+        dict(name='Szymon Judasz', title=N_('Developer')),
+        dict(name='Ville Skyttä', title=N_('Developer')),
+        dict(name='jm4R', title=N_('Developer')),
         dict(name='Igor Galarraga', title=N_('Developer')),
         dict(name='Stanislaw Halik', title=N_('Developer')),
         dict(name='Virgil Dupras', title=N_('Developer')),
         dict(name='Barry Roberts', title=N_('Developer')),
+        dict(name='wsdfhjxc', title=N_('Developer')),
+        dict(name='林博仁(Buo-ren Lin)', title=N_('Developer')),
+        dict(name='Guo Yunhe', title=N_('Developer')),
+        dict(name='cclauss', title=N_('Developer')),
         dict(name='Stefan Naewe', title=N_('Developer')),
-        dict(name='Ville Skyttä', title=N_('Developer')),
+        dict(name='Victor Nepveu', title=N_('Developer')),
         dict(name='Benedict Lee', title=N_('Developer')),
+        dict(name='Filip Danilović', title=N_('Developer')),
+        dict(name='NotSqrt', title=N_('Developer')),
+        dict(name='Pavel Rehak', title=N_('Developer')),
         dict(name='Steffen Prohaska', title=N_('Developer')),
+        dict(name='Tim Brown', title=N_('Developer')),
         dict(name='Michael Geddes', title=N_('Developer')),
         dict(name='Rustam Safin', title=N_('Developer')),
+        dict(name='Alex Gulyás', title=N_('Developer')),
         dict(name='David Martínez Martí', title=N_('Developer')),
+        dict(name='Hualiang Xie', title=N_('Developer')),
         dict(name='Justin Lecher', title=N_('Developer')),
+        dict(name='Kai Krakow', title=N_('Developer')),
         dict(name='Karl Bielefeldt', title=N_('Developer')),
         dict(name='Marco Costalba', title=N_('Developer')),
         dict(name='Michael Homer', title=N_('Developer')),
+        dict(name='Sebastian Schuberth', title=N_('Developer')),
         dict(name='Sven Claussner', title=N_('Developer')),
+        dict(name='bsomers', title=N_('Developer')),
+        dict(name='real', title=N_('Developer')),
         dict(name='v.paritskiy', title=N_('Developer')),
+        dict(name='0xflotus', title=N_('Developer')),
+        dict(name='AJ Bagwell', title=N_('Developer')),
         dict(name='Adrien be', title=N_('Developer')),
+        dict(name='Andrej', title=N_('Developer')),
+        dict(name='Arthur Coelho', title=N_('Developer')),
         dict(name='Audrius Karabanovas', title=N_('Developer')),
+        dict(name='Barrett Lowe', title=N_('Developer')),
         dict(name='Ben Boeckel', title=N_('Developer')),
         dict(name='Boris W', title=N_('Developer')),
         dict(name='Charles', title=N_('Developer')),
@@ -262,13 +310,18 @@ def authors_text():
         dict(name='Daniel King', title=N_('Developer')),
         dict(name='Daniel Pavel', title=N_('Developer')),
         dict(name='David Zumbrunnen', title=N_('Developer')),
+        dict(name='Felipe Morales', title=N_('Developer')),
         dict(name='George Vasilakos', title=N_('Developer')),
         dict(name='Ilya Tumaykin', title=N_('Developer')),
         dict(name='Iulian Udrea', title=N_('Developer')),
         dict(name='Jake Biesinger', title=N_('Developer')),
+        dict(name='Jakub Szymański', title=N_('Developer')),
         dict(name='Jamie Pate', title=N_('Developer')),
+        dict(name='Jean-Francois Dagenais', title=N_('Developer')),
+        dict(name='Joachim Lusiardi', title=N_('Developer')),
         dict(name='Karthik Manamcheri', title=N_('Developer')),
         dict(name='Kelvie Wong', title=N_('Developer')),
+        dict(name='Kyle', title=N_('Developer')),
         dict(name='Maciej Filipiak', title=N_('Developer')),
         dict(name='Maicon D. Filippsen', title=N_('Developer')),
         dict(name='Markus Heidelberg', title=N_('Developer')),
@@ -279,27 +332,33 @@ def authors_text():
         dict(name='Paul Hildebrandt', title=N_('Developer')),
         dict(name='Paul Weingardt', title=N_('Developer')),
         dict(name='Paulo Fidalgo', title=N_('Developer')),
+        dict(name='Petr Gladkikh', title=N_('Developer')),
         dict(name='Philip Stark', title=N_('Developer')),
+        dict(name='Radek Postołowicz', title=N_('Developer')),
         dict(name='Rainer Müller', title=N_('Developer')),
+        dict(name='Ricardo J. Barberis', title=N_('Developer')),
         dict(name='Rolando Espinoza', title=N_('Developer')),
         dict(name="Samsul Ma'arif", title=N_('Developer')),
         dict(name='Sebastian Brass', title=N_('Developer')),
-        dict(name='Sebastian Schuberth', title=N_('Developer')),
+        dict(name='Simon Peeters', title=N_('Developer')),
+        dict(name='Stephen', title=N_('Developer')),
         dict(name='Vaibhav Sagar', title=N_('Developer')),
         dict(name='Ved Vyas', title=N_('Developer')),
         dict(name='Voicu Hodrea', title=N_('Developer')),
         dict(name='Wesley Wong', title=N_('Developer')),
         dict(name='Wolfgang Ocker', title=N_('Developer')),
-        dict(name='ZH', title=N_('Developer')),
-        dict(name='aj-bagwell', title=N_('Developer')),
+        dict(name='Zhang Han', title=N_('Developer')),
+        dict(name='beauxq', title=N_('Developer')),
+        dict(name='ochristi', title=N_('Developer')),
     )
     bug_url = 'https://github.com/git-cola/git-cola/issues'
     bug_link = link(bug_url, bug_url)
-    prelude = N_('''
+    scope = dict(bug_link=bug_link)
+    prelude = N_("""
         <br>
         Please use %(bug_link)s to report issues.
         <br>
-    ''') % locals()
+    """) % scope
 
     return contributors_text(authors, prelude=prelude)
 
@@ -307,41 +366,78 @@ def authors_text():
 def translators_text():
     palette = QtGui.QPalette()
     contact = N_('Email contributor')
-    email = lambda addr: mailto(addr, contact, palette)
 
     translators = (
+        # See the `generate-about` script in the "todo" branch.
+        # vim :read! ./Meta/generate-about --translators
         dict(name='Ｖ字龍(Vdragon)',
              title=N_('Traditional Chinese (Taiwan) translation'),
              email=mailto('Vdragon.Taiwan@gmail.com', contact, palette)),
+        dict(name='Pavel Rehak',
+             title=N_('Czech translation')),
+        dict(name='Victorhck',
+             title=N_('Spanish translation')),
         dict(name='Vitor Lobo',
              title=N_('Brazilian translation')),
+        dict(name='Zhang Han',
+             title=N_('Simplified Chinese translation')),
+        dict(name='Igor Kopach',
+             title=N_('Ukranian translation')),
+        dict(name='Barış ÇELİK',
+             title=N_('Turkish translation')),
+        dict(name='Guo Yunhe',
+             title=N_('Simplified Chinese translation')),
         dict(name='Minarto Margoliono',
              title=N_('Indonesian translation')),
-        dict(name='Zhang Han',
-             title=N_('Chinese translation')),
+        dict(name='Rafael Nascimento',
+             title=N_('Brazilian translation')),
+        dict(name='Shun Sakai',
+             title=N_('Japanese translation')),
         dict(name='Sven Claussner',
              title=N_('German translation')),
         dict(name='Vaiz',
              title=N_('Russian translation')),
-        dict(name='Barış ÇELİK',
+        dict(name='adlgrbz',
              title=N_('Turkish translation')),
-        dict(name='Łukasz Wojniłowicz',
-             email=email('lukasz.wojnilowicz@gmail.com'),
-             title=N_('Polish translation')),
+        dict(name='fu7mu4',
+             title=N_('Japanese translation')),
+        dict(name='Gyuris Gellért',
+             title=N_('Hungarian translation')),
+        dict(name='Joachim Lusiardi',
+             title=N_('German translation')),
+        dict(name='Kai Krakow',
+             title=N_('German translation')),
+        dict(name='Louis Rousseau',
+             title=N_('French translation')),
+        dict(name='Mickael Albertus',
+             title=N_('French translation')),
         dict(name='Peter Dave Hello',
              title=N_('Traditional Chinese (Taiwan) translation')),
         dict(name='Pilar Molina Lopez',
              title=N_('Spanish translation')),
+        dict(name='Rafael Reuber',
+             title=N_('Brazilian translation')),
+        dict(name='Sabri Ünal',
+             title=N_('Turkish translation')),
         dict(name="Samsul Ma'arif",
              title=N_('Indonesian translation')),
         dict(name='Zeioth',
              title=N_('Spanish translation')),
+        dict(name='balping',
+             title=N_('Hungarian translation')),
+        dict(name='p-bo',
+             title=N_('Czech translation')),
+        dict(name='Łukasz Wojniłowicz',
+             title=N_('Polish translation')),
+        dict(name='林博仁(Buo-ren Lin)',
+             title=N_('Traditional Chinese (Taiwan) translation')),
     )
 
     bug_url = 'https://github.com/git-cola/git-cola/issues'
     bug_link = link(bug_url, bug_url)
+    scope = dict(bug_link=bug_link)
 
-    prelude = N_('''
+    prelude = N_("""
         <br>
             Git Cola has been translated into different languages thanks
             to the help of the individuals listed below.
@@ -364,7 +460,7 @@ def translators_text():
 
         <br>
 
-    ''') % locals()
+    """) % scope
     return contributors_text(translators, prelude=prelude)
 
 
@@ -372,7 +468,7 @@ def show_shortcuts():
     hotkeys_html = resources.doc(N_('hotkeys.html'))
     try:
         from qtpy import QtWebEngineWidgets
-    except ImportError:
+    except (ImportError, qtpy.PythonQtError):
         # redhat disabled QtWebKit in their qt build but don't punish the users
         webbrowser.open_new_tab('file://' + hotkeys_html)
         return
@@ -380,11 +476,11 @@ def show_shortcuts():
     html = core.read(hotkeys_html)
 
     parent = qtutils.active_window()
-    widget = QtWidgets.QDialog()
+    widget = QtWidgets.QDialog(parent)
     widget.setWindowModality(Qt.WindowModal)
     widget.setWindowTitle(N_('Shortcuts'))
 
-    web = QtWebEngineWidgets.QWebEngineView(parent)
+    web = QtWebEngineWidgets.QWebEngineView()
     web.setHtml(html)
 
     layout = qtutils.hbox(defs.no_margin, defs.spacing, web)
