@@ -6,7 +6,6 @@ from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
-from ..compat import odict
 from ..compat import uchr
 from ..i18n import N_
 from ..interaction import Interaction
@@ -21,12 +20,6 @@ from .. import qtutils
 from .text import LineEdit
 
 
-SLASH = '/'
-NAME_LOCAL_BRANCH = N_('Local')
-NAME_REMOTE_BRANCH = N_('Remote')
-NAME_TAGS_BRANCH = N_('Tags')
-
-
 def defer_fn(parent, title, fn, *args, **kwargs):
     return qtutils.add_action(parent, title, partial(fn, *args, **kwargs))
 
@@ -46,15 +39,12 @@ def add_branch_to_menu(menu, branch, remote_branch, remote, upstream, fn):
 class AsyncGitActionTask(qtutils.Task):
     """Run git action asynchronously"""
 
-    def __init__(self, parent, git_helper, action, args, kwarg,
-                 refresh_tree, update_remotes):
+    def __init__(self, parent, git_helper, action, args, kwarg):
         qtutils.Task.__init__(self, parent)
         self.git_helper = git_helper
         self.action = action
         self.args = args
         self.kwarg = kwarg
-        self.refresh_tree = refresh_tree
-        self.update_remotes = update_remotes
 
     def task(self):
         """Runs action and captures the result"""
@@ -63,6 +53,8 @@ class AsyncGitActionTask(qtutils.Task):
 
 
 class BranchesWidget(QtWidgets.QFrame):
+    updated = Signal()
+
     def __init__(self, context, parent):
         QtWidgets.QFrame.__init__(self, parent)
         self.model = model = context.model
@@ -101,7 +93,9 @@ class BranchesWidget(QtWidgets.QFrame):
         qtutils.connect_button(self.filter_button, self.toggle_filter)
         qtutils.connect_button(
             self.sort_order_button, cmds.run(cmds.CycleReferenceSort, context))
-        model.add_observer(model.message_ref_sort_changed, self.refresh)
+
+        self.updated.connect(self.refresh, Qt.QueuedConnection)
+        model.add_observer(model.message_refs_updated, self.updated.emit)
 
     def toggle_filter(self):
         shown = not self.filter_widget.isVisible()
@@ -120,14 +114,15 @@ class BranchesWidget(QtWidgets.QFrame):
         self.tree.refresh()
 
 
+# pylint: disable=too-many-ancestors
 class BranchesTreeWidget(standard.TreeWidget):
     updated = Signal()
 
     def __init__(self, context, parent=None):
         standard.TreeWidget.__init__(self, parent)
 
+        model = context.model
         self.context = context
-        self.main_model = model = context.model
 
         self.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
         self.setHeaderHidden(True)
@@ -136,7 +131,7 @@ class BranchesTreeWidget(standard.TreeWidget):
         self.setExpandsOnDoubleClick(False)
 
         self.tree_helper = BranchesTreeHelper()
-        self.git_helper = GitHelper(model.git)
+        self.git_helper = GitHelper(context)
         self.current_branch = None
 
         self.runtask = qtutils.RunTask(parent=self)
@@ -146,6 +141,7 @@ class BranchesTreeWidget(standard.TreeWidget):
         model.add_observer(model.message_updated, self.updated.emit)
 
         # Expand items when they are clicked
+        # pylint: disable=no-member
         self.clicked.connect(self._toggle_expanded)
 
         # Checkout branch when double clicked
@@ -154,31 +150,26 @@ class BranchesTreeWidget(standard.TreeWidget):
     def refresh(self):
         if not self._active:
             return
-        model = self.main_model
+        model = self.context.model
         self.current_branch = model.currentbranch
 
         states = self.save_tree_state()
-
-        local_dict = self.tree_helper.group_branches(
-            model.local_branches, SLASH)
-
-        remote_dict = self.tree_helper.group_branches(
-            model.remote_branches, SLASH)
-
-        tags_dict = self.tree_helper.group_branches(model.tags, SLASH)
-
         ellipsis = icons.ellipsis()
-        local = self.tree_helper.create_top_level_item(
-            NAME_LOCAL_BRANCH, local_dict,
-            icon=icons.branch(), ellipsis=ellipsis)
 
-        remote = self.tree_helper.create_top_level_item(
-            NAME_REMOTE_BRANCH, remote_dict,
-            icon=icons.branch(), ellipsis=ellipsis)
+        local_tree = create_tree_entries(model.local_branches)
+        local_tree.basename = N_('Local')
+        local = create_toplevel_item(
+            local_tree, icon=icons.branch(), ellipsis=ellipsis)
 
-        tags = self.tree_helper.create_top_level_item(
-            NAME_TAGS_BRANCH, tags_dict,
-            icon=icons.tag(), ellipsis=ellipsis)
+        remote_tree = create_tree_entries(model.remote_branches)
+        remote_tree.basename = N_('Remote')
+        remote = create_toplevel_item(
+            remote_tree, icon=icons.branch(), ellipsis=ellipsis)
+
+        tags_tree = create_tree_entries(model.tags)
+        tags_tree.basename = N_('Tags')
+        tags = create_toplevel_item(
+            tags_tree, icon=icons.tag(), ellipsis=ellipsis)
 
         self.clear()
         self.addTopLevelItems([local, remote, tags])
@@ -202,12 +193,12 @@ class BranchesTreeWidget(standard.TreeWidget):
         selected = self.selected_item()
         if not selected:
             return
-        root = self.tree_helper.get_root(selected)
-
-        if selected.childCount() > 0 or not root:
+        # Only allow actions on leaf nodes that have a valid refname.
+        if not selected.refname:
             return
 
-        full_name = self.tree_helper.get_full_name(selected, SLASH)
+        root = get_toplevel_item(selected)
+        full_name = selected.refname
         menu = qtutils.create_menu(N_('Actions'), self)
 
         # all branches except current the current branch
@@ -215,7 +206,7 @@ class BranchesTreeWidget(standard.TreeWidget):
             menu.addAction(qtutils.add_action(
                 menu, N_('Checkout'), self.checkout_action))
             # remote branch
-            if NAME_REMOTE_BRANCH == root.name:
+            if root.name == N_('Remote'):
                 label = N_('Checkout as new branch')
                 action = self.checkout_new_branch_action
                 menu.addAction(qtutils.add_action(menu, label, action))
@@ -227,9 +218,9 @@ class BranchesTreeWidget(standard.TreeWidget):
             menu.addAction(merge_menu_action)
 
         # local and remote branch
-        if NAME_TAGS_BRANCH != root.name:
+        if root.name != N_('Tags'):
             # local branch
-            if NAME_LOCAL_BRANCH == root.name:
+            if root.name == N_('Local'):
 
                 remote = gitcmds.tracked_branch(context, full_name)
                 if remote is not None:
@@ -255,7 +246,7 @@ class BranchesTreeWidget(standard.TreeWidget):
             # not current branch
             if full_name != self.current_branch:
                 delete_label = N_('Delete Branch')
-                if NAME_REMOTE_BRANCH == root.name:
+                if root.name == N_('Remote'):
                     delete_label = N_('Delete Remote Branch')
 
                 delete_menu_action = qtutils.add_action(
@@ -266,7 +257,7 @@ class BranchesTreeWidget(standard.TreeWidget):
                 menu.addAction(delete_menu_action)
 
         # manage upstreams for local branches
-        if root.name == NAME_LOCAL_BRANCH:
+        if root.name == N_('Local'):
             upstream_menu = menu.addMenu(N_('Set Upstream Branch'))
             upstream_menu.setIcon(icons.branch())
             self.build_upstream_menu(upstream_menu)
@@ -276,9 +267,8 @@ class BranchesTreeWidget(standard.TreeWidget):
     def build_upstream_menu(self, menu):
         """Build the "Set Upstream Branch" sub-menu"""
         context = self.context
-        model = self.main_model
-        selected_item = self.selected_item()
-        selected_branch = self.tree_helper.get_full_name(selected_item, SLASH)
+        model = context.model
+        selected_branch = self.selected_refname()
         remote = None
         upstream = None
 
@@ -355,10 +345,10 @@ class BranchesTreeWidget(standard.TreeWidget):
         context = self.context
         current_branch = self.current_branch
         top_item = self.topLevelItem(0)
-        item = self.tree_helper.find_child(top_item, current_branch)
+        item = find_by_refname(top_item, current_branch)
 
         if item is not None:
-            self.tree_helper.expand_from_item(item)
+            expand_item_parents(item)
             item.setIcon(0, icons.star())
 
             tracked_branch = gitcmds.tracked_branch(context, current_branch)
@@ -383,13 +373,11 @@ class BranchesTreeWidget(standard.TreeWidget):
                 if status_str:
                     item.setText(0, '%s\t%s' % (item.text(0), status_str))
 
-    def git_action_async(self, action, args, kwarg=None, refresh_tree=False,
-                         update_remotes=False):
+    def git_action_async(self, action, args, kwarg=None):
         if kwarg is None:
             kwarg = {}
-        task = AsyncGitActionTask(self, self.git_helper, action, args, kwarg,
-                                  refresh_tree, update_remotes)
-        progress = standard.ProgressDialog(
+        task = AsyncGitActionTask(self, self.git_helper, action, args, kwarg)
+        progress = standard.progress(
             N_('Executing action %s') % action, N_('Updating'), self)
         self.runtask.start(task, progress=progress,
                            finish=self.git_action_completed)
@@ -397,15 +385,11 @@ class BranchesTreeWidget(standard.TreeWidget):
     def git_action_completed(self, task):
         status, out, err = task.result
         self.git_helper.show_result(task.action, status, out, err)
-        if task.refresh_tree:
-            self.refresh()
-        if task.update_remotes:
-            model = self.main_model
-            model.update_remotes()
+        self.context.model.update_refs()
 
     def push_action(self):
         context = self.context
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
+        branch = self.selected_refname()
         remote_branch = gitcmds.tracked_branch(context, branch)
         if remote_branch:
             remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
@@ -415,71 +399,67 @@ class BranchesTreeWidget(standard.TreeWidget):
                 self.git_action_async('push', [remote, branch_name])
 
     def rename_action(self):
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        new_branch = qtutils.prompt(
+        branch = self.selected_refname()
+        new_branch, ok = qtutils.prompt(
             N_('Enter New Branch Name'),
             title=N_('Rename branch'), text=branch)
-        if new_branch[1] is True and new_branch[0]:
-            self.git_action_async('rename', [branch, new_branch[0]],
-                                  refresh_tree=True)
+        if ok and new_branch:
+            self.git_action_async('rename', [branch, new_branch])
 
     def pull_action(self):
         context = self.context
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
+        branch = self.selected_refname()
+        if not branch:
+            return
         remote_branch = gitcmds.tracked_branch(context, branch)
         if remote_branch:
             remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
             if remote and branch_name:
-                self.git_action_async(
-                    'pull', [remote, branch_name], refresh_tree=True)
+                self.git_action_async('pull', [remote, branch_name])
 
     def delete_action(self):
-        title = N_('Delete Branch')
-        question = N_('Delete selected branch?')
-        info = N_('The branch will be no longer available.')
-        ok_btn = N_('Delete Branch')
+        branch = self.selected_refname()
+        if not branch or branch == self.current_branch:
+            return
 
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
+        remote = False
+        root = get_toplevel_item(self.selected_item())
+        if root.name == N_('Remote'):
+            remote = True
 
-        if (branch != self.current_branch and
-                Interaction.confirm(title, question, info, ok_btn)):
-            remote = False
-            root = self.tree_helper.get_root(self.selected_item())
-            if NAME_REMOTE_BRANCH == root.name:
-                remote = True
-
-            if remote:
-                remote, branch_name = gitcmds.parse_remote_branch(branch)
-                if remote and branch_name:
-                    self.git_action_async(
-                        'delete_remote', [remote, branch_name],
-                        update_remotes=True)
-            else:
-                self.git_action_async('delete_local', [branch])
+        if remote:
+            remote, branch = gitcmds.parse_remote_branch(branch)
+            if remote and branch:
+                cmds.do(cmds.DeleteRemoteBranch, self.context, remote, branch)
+        else:
+            cmds.do(cmds.DeleteBranch, self.context, branch)
 
     def merge_action(self):
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-
-        if branch != self.current_branch:
-            self.git_action_async('merge', [branch], refresh_tree=True)
+        branch = self.selected_refname()
+        if branch and branch != self.current_branch:
+            self.git_action_async('merge', [branch])
 
     def checkout_action(self):
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        if branch != self.current_branch:
+        branch = self.selected_refname()
+        if branch and branch != self.current_branch:
             self.git_action_async('checkout', [branch])
 
     def checkout_new_branch_action(self):
-        branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        if branch != self.current_branch:
+        branch = self.selected_refname()
+        if branch and branch != self.current_branch:
             _, new_branch = gitcmds.parse_remote_branch(branch)
             self.git_action_async('checkout', ['-b', new_branch, branch])
 
+    def selected_refname(self):
+        return getattr(self.selected_item(), 'refname', None)
+
 
 class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
 
-    def __init__(self, name, icon=None):
+    def __init__(self, name, refname=None, icon=None):
         QtWidgets.QTreeWidgetItem.__init__(self)
         self.name = name
+        self.refname = refname
         self.setText(0, name)
         self.setToolTip(0, name)
         if icon is not None:
@@ -495,95 +475,164 @@ class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
         return 1
 
 
-class BranchesTreeHelper(object):
-    @staticmethod
-    def group_branches(list_branches, separator_char):
-        """Convert a list of delimited strings to a nested tree dict"""
-        result = odict()
-        for item in list_branches:
-            tree = result
-            for part in item.split(separator_char):
-                tree = tree.setdefault(part, odict())
+class TreeEntry(object):
+    """Tree representation for the branches widget
 
-        return result
+    The branch widget UI displays the basename.  For intermediate names, e.g.
+    "xxx" in the "xxx/abc" and "xxx/def" branches, the 'refname' will be None.
+    'children' contains a list of TreeEntry, and is empty when refname is
+    defined.
 
-    @staticmethod
-    def create_top_level_item(name, dict_children,
-                              icon=None, ellipsis=None):
-        """Create a top level tree item and its children """
+    """
+    def __init__(self, basename, refname, children):
+        self.basename = basename
+        self.refname = refname
+        self.children = children
 
-        def create_children(grouped_branches):
-            """Create children items for a tree item"""
-            result = []
-            for k, v in grouped_branches.items():
-                item = BranchTreeWidgetItem(k, icon=icon)
-                item.addChildren(create_children(v))
 
-                if item.childCount() > 0 and ellipsis is not None:
-                    item.setIcon(0, ellipsis)
+def create_tree_entries(names):
+    """Create a nested tree structure with a single root TreeEntry.
 
-                result.append(item)
+    When names == ['xxx/abc', 'xxx/def'] the result will be::
 
+        TreeEntry(
+            basename=None,
+            refname=None,
+            children=[
+                TreeEntry(
+                    basename='xxx',
+                    refname=None,
+                    children=[
+                        TreeEntry(
+                            basename='abc',
+                            refname='xxx/abc',
+                            children=[]
+                        ),
+                        TreeEntry(
+                            basename='def',
+                            refname='xxx/def',
+                            children=[]
+                        )
+                    ]
+                )
+            ]
+        )
+
+    """
+    # Phase 1: build a nested dictionary representing the intermediate
+    # names in the branches.  e.g. {'xxx': {'abc': {}, 'def': {}}}
+    tree_names = create_name_dict(names)
+
+    # Loop over the names again, this time we'll create tree entries
+    entries = {}
+    root = TreeEntry(None, None, [])
+    for item in names:
+        cur_names = tree_names
+        cur_entries = entries
+        tree = root
+        children = root.children
+        for part in item.split('/'):
+            if cur_names[part]:
+                # This has children
+                try:
+                    tree, _ = cur_entries[part]
+                except KeyError:
+                    # New entry
+                    tree = TreeEntry(part, None, [])
+                    cur_entries[part] = (tree, {})
+                    # Append onto the parent children list only once
+                    children.append(tree)
+            else:
+                # This is the actual branch
+                tree = TreeEntry(part, item, [])
+                children.append(tree)
+                cur_entries[part] = (tree, {})
+
+            # Advance into the nested child list
+            children = tree.children
+            # Advance into the inner dict
+            cur_names = cur_names[part]
+            _, cur_entries = cur_entries[part]
+
+    return root
+
+
+def create_name_dict(names):
+    # Phase 1: build a nested dictionary representing the intermediate
+    # names in the branches.  e.g. {'xxx': {'abc': {}, 'def': {}}}
+    tree_names = {}
+    for item in names:
+        part_names = tree_names
+        for part in item.split('/'):
+            # Descend into the inner names dict.
+            part_names = part_names.setdefault(part, {})
+    return tree_names
+
+
+def create_toplevel_item(tree, icon=None, ellipsis=None):
+    """Create a top-level BranchTreeWidgetItem and its children"""
+
+    item = BranchTreeWidgetItem(tree.basename, icon=ellipsis)
+    children = create_tree_items(tree.children, icon=icon, ellipsis=ellipsis)
+    if children:
+        item.addChildren(children)
+    return item
+
+
+def create_tree_items(entries, icon=None, ellipsis=None):
+    """Create children items for a tree item"""
+    result = []
+    for tree in entries:
+        item = BranchTreeWidgetItem(
+            tree.basename, refname=tree.refname, icon=icon)
+        children = create_tree_items(
+            tree.children, icon=icon, ellipsis=ellipsis)
+        if children:
+            item.addChildren(children)
+            if ellipsis is not None:
+                item.setIcon(0, ellipsis)
+        result.append(item)
+
+    return result
+
+
+def expand_item_parents(item):
+    """Expand tree parents from item"""
+    parent = item.parent()
+    while parent is not None:
+        parent.setExpanded(True)
+        parent = parent.parent()
+
+
+def find_by_refname(item, refname):
+    """Find child by full name recursive"""
+    result = None
+
+    for i in range(item.childCount()):
+        child = item.child(i)
+        if child.refname and child.refname == refname:
+            return child
+
+        result = find_by_refname(child, refname)
+        if result is not None:
             return result
 
-        branch = BranchTreeWidgetItem(name, icon=ellipsis)
-        branch.addChildren(create_children(dict_children))
+    return result
 
-        return branch
 
-    @staticmethod
-    def get_root(item):
-        """Returns top level item from an item"""
-        parents = [item]
-        parent = item.parent()
+def get_toplevel_item(item):
+    """Returns top-most item found by traversing up the specified item"""
+    parents = [item]
+    parent = item.parent()
 
-        while parent is not None:
-            parents.append(parent)
-            parent = parent.parent()
+    while parent is not None:
+        parents.append(parent)
+        parent = parent.parent()
 
-        return parents[len(parents) - 1]
+    return parents[-1]
 
-    @staticmethod
-    def get_full_name(item, join_char):
-        """Returns item full name generated by iterating over
-        parents and joining their names with 'join_char'"""
-        parents = [item.name]
-        parent = item.parent()
 
-        while parent is not None:
-            parents.append(parent.name)
-            parent = parent.parent()
-
-        result = join_char.join(reversed(parents))
-
-        return result[result.find(join_char) + 1:]
-
-    @staticmethod
-    def expand_from_item(item):
-        """Expand tree parents from item"""
-        parent = item.parent()
-
-        while parent is not None:
-            parent.setExpanded(True)
-            parent = parent.parent()
-
-    def find_child(self, top_level_item, name):
-        """Find child by full name recursive"""
-        result = None
-
-        for i in range(top_level_item.childCount()):
-            child = top_level_item.child(i)
-            full_name = self.get_full_name(child, SLASH)
-
-            if full_name == name:
-                result = child
-                return result
-            else:
-                result = self.find_child(child, name)
-                if result is not None:
-                    return result
-
-        return result
+class BranchesTreeHelper(object):
 
     def load_state(self, item, state):
         """Load expanded items from a dict"""
@@ -609,8 +658,9 @@ class BranchesTreeHelper(object):
 
 class GitHelper(object):
 
-    def __init__(self, git):
-        self.git = git
+    def __init__(self, context):
+        self.context = context
+        self.git = context.git
 
     def log(self, origin):
         return self.git.log(origin, oneline=True)
@@ -621,12 +671,6 @@ class GitHelper(object):
     def pull(self, remote, branch):
         return self.git.pull(remote, branch, no_ff=True, verbose=True)
 
-    def delete_remote(self, remote, branch):
-        return self.git.push(remote, branch, delete=True)
-
-    def delete_local(self, branch):
-        return self.git.branch(branch, D=True)
-
     def merge(self, branch):
         return self.git.merge(branch, no_commit=True)
 
@@ -658,6 +702,7 @@ class BranchesFilterWidget(QtWidgets.QWidget):
         self.setLayout(self.main_layout)
 
         text = self.text
+        # pylint: disable=no-member
         text.textChanged.connect(self.apply_filter)
         self.tree.updated.connect(self.apply_filter, type=Qt.QueuedConnection)
 
