@@ -5,24 +5,20 @@ from qtpy import QtCore
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
+from ..i18n import N_
+from ..interaction import Interaction
+from ..qtutils import get
 from .. import gitcmds
 from .. import icons
 from .. import qtutils
-from ..i18n import N_
-from ..interaction import Interaction
-from ..models import main
 from . import defs
 from . import completion
-from .standard import Dialog
-from .standard import ProgressDialog
-from .text import LineEdit
+from . import standard
 
 
-def create_new_branch(revision='', settings=None):
+def create_new_branch(context, revision='', settings=None):
     """Launches a dialog for creating a new branch"""
-    model = main.MainModel()
-    model.update_status()
-    view = CreateBranchDialog(model, settings=settings,
+    view = CreateBranchDialog(context, settings=settings,
                               parent=qtutils.active_window())
     if revision:
         view.set_revision(revision)
@@ -31,8 +27,8 @@ def create_new_branch(revision='', settings=None):
 
 
 class CreateOpts(object):
-    def __init__(self, model):
-        self.model = model
+    def __init__(self, context):
+        self.context = context
         self.reset = False
         self.track = False
         self.fetch = True
@@ -55,43 +51,45 @@ class CreateThread(QtCore.QThread):
         reset = self.opts.reset
         checkout = self.opts.checkout
         track = self.opts.track
-        model = self.opts.model
+        context = self.opts.context
+        git = context.git
+        model = context.model
         results = []
         status = 0
 
         if track and '/' in revision:
             remote = revision.split('/', 1)[0]
-            status, out, err = model.git.fetch(remote)
+            status, out, err = git.fetch(remote)
             self.command.emit(status, out, err)
             results.append(('fetch', status, out, err))
 
         if status == 0:
-            status, out, err = model.create_branch(branch, revision,
-                                                   force=reset,
-                                                   track=track)
+            status, out, err = model.create_branch(
+                branch, revision, force=reset, track=track)
             self.command.emit(status, out, err)
 
         results.append(('branch', status, out, err))
         if status == 0 and checkout:
-            status, out, err = model.git.checkout(branch)
+            status, out, err = git.checkout(branch)
             self.command.emit(status, out, err)
             results.append(('checkout', status, out, err))
 
-        main.model().update_status()
+        model.update_status()
         self.result.emit(results)
 
 
-class CreateBranchDialog(Dialog):
+class CreateBranchDialog(standard.Dialog):
     """A dialog for creating branches."""
 
-    def __init__(self, model, settings=None, parent=None):
-        Dialog.__init__(self, parent=parent)
+    def __init__(self, context, settings=None, parent=None):
+        standard.Dialog.__init__(self, parent=parent)
         self.setWindowTitle(N_('Create Branch'))
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
 
-        self.model = model
-        self.opts = CreateOpts(model)
+        self.context = context
+        self.model = context.model
+        self.opts = CreateOpts(context)
         self.thread = CreateThread(self.opts, self)
 
         self.progress = None
@@ -99,13 +97,16 @@ class CreateBranchDialog(Dialog):
         self.branch_name_label = QtWidgets.QLabel()
         self.branch_name_label.setText(N_('Branch Name'))
 
-        self.branch_name = LineEdit()
+        self.branch_name = completion.GitCreateBranchLineEdit(context)
+        self.branch_validator = completion.BranchValidator(
+            context.git, parent=self.branch_name)
+        self.branch_name.setValidator(self.branch_validator)
 
         self.rev_label = QtWidgets.QLabel()
         self.rev_label.setText(N_('Starting Revision'))
 
-        self.revision = completion.GitRefLineEdit()
-        current = gitcmds.current_branch()
+        self.revision = completion.GitRefLineEdit(context)
+        current = gitcmds.current_branch(context)
         if current:
             self.revision.setText(current)
 
@@ -176,9 +177,9 @@ class CreateBranchDialog(Dialog):
                                                  qtutils.STRETCH)
 
         self.buttons_layout = qtutils.hbox(defs.margin, defs.spacing,
+                                           self.close_button,
                                            qtutils.STRETCH,
-                                           self.create_button,
-                                           self.close_button)
+                                           self.create_button)
 
         self.main_layout = qtutils.vbox(defs.margin, defs.spacing,
                                         self.branch_name_layout,
@@ -192,53 +193,57 @@ class CreateBranchDialog(Dialog):
         qtutils.add_close_action(self)
         qtutils.connect_button(self.close_button, self.close)
         qtutils.connect_button(self.create_button, self.create_branch)
-        qtutils.connect_button(self.local_radio, self.display_model)
-        qtutils.connect_button(self.remote_radio, self.display_model)
-        qtutils.connect_button(self.tag_radio, self.display_model)
+        qtutils.connect_toggle(self.local_radio, self.display_model)
+        qtutils.connect_toggle(self.remote_radio, self.display_model)
+        qtutils.connect_toggle(self.tag_radio, self.display_model)
 
         branches = self.branch_list
+        # pylint: disable=no-member
         branches.itemSelectionChanged.connect(self.branch_item_changed)
 
         thread = self.thread
-        thread.command.connect(self.thread_command, type=Qt.QueuedConnection)
+        thread.command.connect(Interaction.log_status,
+                               type=Qt.QueuedConnection)
         thread.result.connect(self.thread_result, type=Qt.QueuedConnection)
 
-        self.init_state(settings, self.resize, 555, 333)
+        self.init_size(settings=settings, parent=parent)
         self.display_model()
 
     def set_revision(self, revision):
         self.revision.setText(revision)
 
     def getopts(self):
-        self.opts.revision = self.revision.value()
-        self.opts.branch = self.branch_name.value()
-        self.opts.checkout = self.checkout_checkbox.isChecked()
-        self.opts.reset = self.reset_radio.isChecked()
-        self.opts.fetch = self.fetch_checkbox.isChecked()
-        self.opts.track = self.remote_radio.isChecked()
+        self.opts.revision = get(self.revision)
+        self.opts.branch = get(self.branch_name)
+        self.opts.checkout = get(self.checkout_checkbox)
+        self.opts.reset = get(self.reset_radio)
+        self.opts.fetch = get(self.fetch_checkbox)
+        self.opts.track = get(self.remote_radio)
 
     def create_branch(self):
         """Creates a branch; called by the "Create Branch" button"""
+        context = self.context
         self.getopts()
         revision = self.opts.revision
         branch = self.opts.branch
-        no_update = self.no_update_radio.isChecked()
-        ffwd_only = self.ffwd_only_radio.isChecked()
-        existing_branches = gitcmds.branch_list()
+        no_update = get(self.no_update_radio)
+        ffwd_only = get(self.ffwd_only_radio)
+        existing_branches = gitcmds.branch_list(context)
         check_branch = False
 
         if not branch or not revision:
-            qtutils.critical(N_('Missing Data'),
-                             N_('Please provide both a branch '
-                                'name and revision expression.'))
+            Interaction.critical(
+                N_('Missing Data'),
+                N_('Please provide both a branch '
+                   'name and revision expression.'))
             return
         if branch in existing_branches:
             if no_update:
                 msg = N_('Branch "%s" already exists.') % branch
-                qtutils.critical(N_('Branch Exists'), msg)
+                Interaction.critical(N_('Branch Exists'), msg)
                 return
             # Whether we should prompt the user for lost commits
-            commits = gitcmds.rev_list_range(revision, branch)
+            commits = gitcmds.rev_list_range(context, revision, branch)
             check_branch = bool(commits)
 
         if check_branch:
@@ -246,7 +251,7 @@ class CreateBranchDialog(Dialog):
                       'will lose commits.') %
                    dict(branch=branch, revision=revision))
             if ffwd_only:
-                qtutils.critical(N_('Branch Exists'), msg)
+                Interaction.critical(N_('Branch Exists'), msg)
                 return
             lines = [msg]
             for idx, commit in enumerate(commits):
@@ -264,37 +269,32 @@ class CreateBranchDialog(Dialog):
             info_text = (N_('Reset "%(branch)s" to "%(revision)s"?') %
                          dict(branch=branch, revision=revision))
 
-            if not qtutils.confirm(N_('Reset Branch?'),
-                                   '\n'.join(lines),
-                                   info_text,
-                                   N_('Reset Branch'),
-                                   default=False,
-                                   icon=icons.undo()):
+            if not Interaction.confirm(
+                    N_('Reset Branch?'), '\n'.join(lines), info_text,
+                    N_('Reset Branch'), default=False, icon=icons.undo()):
                 return
 
         title = N_('Create Branch')
         label = N_('Updating')
-        self.progress = ProgressDialog(title, label, self)
+        self.progress = standard.progress(title, label, self)
         self.progress.show()
         self.thread.start()
 
-    def thread_command(self, status, out, err):
-        Interaction.log_status(status, out, err)
-
     def thread_result(self, results):
         self.progress.hide()
         del self.progress
 
-        for (cmd, status, out, err) in results:
+        for (cmd, status, _, _) in results:
             if status != 0:
                 Interaction.critical(
-                        N_('Error Creating Branch'),
-                        (N_('"%(command)s" returned exit status "%(status)d"') %
-                         dict(command='git '+cmd, status=status)))
+                    N_('Error Creating Branch'),
+                    (N_('"%(command)s" returned exit status "%(status)d"')
+                     % dict(command='git '+cmd, status=status)))
                 return
 
         self.accept()
 
+    # pylint: disable=unused-argument
     def branch_item_changed(self, *rest):
         """This callback is called when the branch selection changes"""
         # When the branch selection changes then we should update
@@ -307,7 +307,7 @@ class CreateBranchDialog(Dialog):
         self.revision.setText(remote_branch)
 
         # Set the branch field if we're branching from a remote branch.
-        if not self.remote_radio.isChecked():
+        if not get(self.remote_radio):
             return
         branch = gitcmds.strip_remote(self.model.remotes, remote_branch)
         if branch == 'HEAD':
@@ -324,9 +324,16 @@ class CreateBranchDialog(Dialog):
     def branch_sources(self):
         """Get the list of items for populating the branch root list.
         """
-        if self.local_radio.isChecked():
-            return self.model.local_branches
-        elif self.remote_radio.isChecked():
-            return self.model.remote_branches
-        elif self.tag_radio.isChecked():
-            return self.model.tags
+        if get(self.local_radio):
+            value = self.model.local_branches
+        elif get(self.remote_radio):
+            value = self.model.remote_branches
+        elif get(self.tag_radio):
+            value = self.model.tags
+        else:
+            value = []
+        return value
+
+    def dispose(self):
+        self.branch_name.dispose()
+        self.revision.dispose()
