@@ -1,6 +1,5 @@
 from __future__ import division, absolute_import, unicode_literals
 
-from qtpy import QtCore
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 
@@ -8,24 +7,25 @@ from .. import cmds
 from .. import icons
 from .. import qtutils
 from ..i18n import N_
+from ..qtutils import get
 from . import defs
 from . import completion
 from . import standard
 from . import text
 
 
-def new_create_tag(name='', ref='', sign=False, settings=None, parent=None):
+def new_create_tag(context, name='', ref='', sign=False,
+                   settings=None, parent=None):
     """Entry point for external callers."""
     opts = TagOptions(name, ref, sign)
-    view = CreateTag(opts, settings=settings, parent=parent)
+    view = CreateTag(context, opts, settings=settings, parent=parent)
     return view
 
 
-def create_tag(name='', ref='', sign=False, settings=None):
+def create_tag(context, name='', ref='', sign=False, settings=None):
     """Entry point for external callers."""
-    view = new_create_tag(name=name, ref=ref, sign=sign,
-                          settings=settings,
-                          parent=qtutils.active_window())
+    view = new_create_tag(context, name=name, ref=ref, sign=sign,
+                          settings=settings, parent=qtutils.active_window())
     view.show()
     view.raise_()
     return view
@@ -42,23 +42,27 @@ class TagOptions(object):
 
 class CreateTag(standard.Dialog):
 
-    def __init__(self, opts, settings=None, parent=None):
+    def __init__(self, context, opts, settings=None, parent=None):
         standard.Dialog.__init__(self, parent=parent)
 
+        self.context = context
+        self.model = model = context.model
         self.opts = opts
 
         self.setWindowTitle(N_('Create Tag'))
         if parent is not None:
-            self.setWindowModality(QtCore.Qt.WindowModal)
+            self.setWindowModality(Qt.WindowModal)
 
         # Tag label
         self.tag_name_label = QtWidgets.QLabel(self)
         self.tag_name_label.setText(N_('Name'))
 
-        self.tag_name = text.HintedLineEdit(N_('vX.Y.Z'), parent=self)
+        self.tag_name = text.HintedLineEdit(context, N_('vX.Y.Z'), parent=self)
         self.tag_name.set_value(opts.name)
         self.tag_name.setToolTip(N_('Specifies the tag name'))
 
+        qtutils.add_completer(self.tag_name, model.tags)
+
         # Sign Tag
         self.sign_label = QtWidgets.QLabel(self)
         self.sign_label.setText(N_('Sign Tag'))
@@ -70,13 +74,14 @@ class CreateTag(standard.Dialog):
         self.tag_msg_label = QtWidgets.QLabel(self)
         self.tag_msg_label.setText(N_('Message'))
 
-        self.tag_msg = text.HintedPlainTextEdit(N_('Tag message...'), self)
+        self.tag_msg = text.HintedPlainTextEdit(
+            context, N_('Tag message...'), self)
         self.tag_msg.setToolTip(N_('Specifies the tag message'))
         # Revision
         self.rev_label = QtWidgets.QLabel(self)
         self.rev_label.setText(N_('Revision'))
 
-        self.revision = completion.GitRefLineEdit()
+        self.revision = completion.GitRefLineEdit(context)
         self.revision.setText(self.opts.ref)
         self.revision.setToolTip(N_('Specifies the SHA-1 to tag'))
         # Buttons
@@ -104,44 +109,19 @@ class CreateTag(standard.Dialog):
         qtutils.connect_button(self.close_button, self.close)
         qtutils.connect_button(self.create_button, self.create_tag)
 
-        self.init_state(settings, self.resize, defs.scale(720), defs.scale(210))
+        self.init_state(settings, self.resize,
+                        defs.scale(720), defs.scale(210))
 
     def create_tag(self):
         """Verifies inputs and emits a notifier tag message."""
 
-        revision = self.revision.value()
-        tag_name = self.tag_name.value()
-        tag_msg = self.tag_msg.value()
-        sign_tag = self.sign_tag.isChecked()
+        context = self.context
+        revision = get(self.revision)
+        tag_name = get(self.tag_name)
+        tag_msg = get(self.tag_msg)
+        sign_tag = get(self.sign_tag)
 
-        if not revision:
-            qtutils.critical(N_('Missing Revision'),
-                             N_('Please specify a revision to tag.'))
-            return
-        elif not tag_name:
-            qtutils.critical(N_('Missing Name'),
-                             N_('Please specify a name for the new tag.'))
-            return
-        elif (sign_tag and not tag_msg and
-                not qtutils.confirm(N_('Missing Tag Message'),
-                                    N_('Tag-signing was requested but the tag '
-                                       'message is empty.'),
-                                    N_('An unsigned, lightweight tag will be '
-                                       'created instead.\n'
-                                       'Create an unsigned tag?'),
-                                    N_('Create Unsigned Tag'),
-                                    default=False, icon=icons.save())):
-            return
-
-        status, output, err = cmds.do(cmds.Tag, tag_name, revision,
-                                      sign=sign_tag, message=tag_msg)
-
-        if status == 0:
-            qtutils.information(N_('Tag Created'),
-                                N_('Created a new tag named "%s"') % tag_name,
-                                details=tag_msg or None)
+        ok = cmds.do(cmds.Tag, context, tag_name, revision,
+                     sign=sign_tag, message=tag_msg)
+        if ok:
             self.close()
-        else:
-            qtutils.critical(N_('Error: could not create tag "%s"') % tag_name,
-                             (N_('git tag returned exit code %s') % status) +
-                             ((output+err) and ('\n\n' + output + err) or ''))
