@@ -60,9 +60,15 @@ class DiffSyntaxHighlighter(QtGui.QSyntaxHighlighter):
         disabled = palette.color(QPalette.Disabled, QPalette.Text)
         header = qtutils.rgb_hex(disabled)
 
+        dark = palette.color(QPalette.Base).lightnessF() < 0.5
+
         self.color_text = qtutils.RGB(cfg.color('text', '030303'))
-        self.color_add = qtutils.RGB(cfg.color('add', 'd2ffe4'))
-        self.color_remove = qtutils.RGB(cfg.color('remove', 'fee0e4'))
+        self.color_add = qtutils.RGB(
+            cfg.color('add', '77aa77' if dark else 'd2ffe4')
+        )
+        self.color_remove = qtutils.RGB(
+            cfg.color('remove', 'aa7777' if dark else 'fee0e4')
+        )
         self.color_header = qtutils.RGB(cfg.color('header', header))
 
         self.diff_header_fmt = qtutils.make_format(fg=self.color_header)
@@ -153,6 +159,7 @@ class DiffSyntaxHighlighter(QtGui.QSyntaxHighlighter):
         self.setCurrentBlockState(state)
 
 
+# pylint: disable=too-many-ancestors
 class DiffTextEdit(VimHintedPlainTextEdit):
 
     def __init__(self, context, parent,
@@ -168,7 +175,7 @@ class DiffTextEdit(VimHintedPlainTextEdit):
         else:
             self.numbers = None
         self.scrollvalue = None
-
+        # pylint: disable=no-member
         self.cursorPositionChanged.connect(self._cursor_changed)
 
     def _cursor_changed(self):
@@ -304,6 +311,7 @@ class DiffLineNumbers(TextDecorator):
         event_rect_bottom = event.rect().bottom()
 
         highlight = self._highlight
+        highlight.setAlphaF(0.3)
         highlight_text = self._highlight_text
         disabled = self._disabled
 
@@ -345,7 +353,7 @@ class DiffLineNumbers(TextDecorator):
             block = block.next()  # pylint: disable=next-method-called
 
 
-class Viewer(QtWidgets.QWidget):
+class Viewer(QtWidgets.QFrame):
     """Text and image diff viewers"""
 
     images_changed = Signal(object)
@@ -374,6 +382,7 @@ class Viewer(QtWidgets.QWidget):
         # Observe images
         images_msg = model.message_images_changed
         model.add_observer(images_msg, self.images_changed.emit)
+        # pylint: disable=no-member
         self.images_changed.connect(self.set_images, type=Qt.QueuedConnection)
 
         # Observe the diff type
@@ -568,7 +577,6 @@ class Options(QtWidgets.QWidget):
 
         self.options = options = qtutils.create_action_button(
             tooltip=N_('Diff Options'), icon=icons.configure())
-        qtutils.hide_button_menu_indicator(options)
 
         self.image_mode = qtutils.combo([
             N_('Side by side'),
@@ -630,6 +638,7 @@ class Options(QtWidgets.QWidget):
         self.widget.update_options()
 
 
+# pylint: disable=too-many-ancestors
 class DiffEditor(DiffTextEdit):
 
     up = Signal()
@@ -668,6 +677,7 @@ class DiffEditor(DiffTextEdit):
 
         selection_model.add_observer(
             selection_model.message_selection_changed, self.updated.emit)
+        # pylint: disable=no-member
         self.updated.connect(self.refresh, type=Qt.QueuedConnection)
         # Update the selection model when the cursor changes
         self.cursorPositionChanged.connect(self._update_line_number)
