@@ -49,12 +49,13 @@ class LogWidget(QtWidgets.QFrame):
         """Append to the end of the log message"""
         if not msg:
             return
+        msg = core.decode(msg)
         cursor = self.output_text.textCursor()
         cursor.movePosition(cursor.End)
         text = self.output_text
         # NOTE: the ':  ' colon-SP-SP suffix is for the syntax highlighter
         prefix = core.decode(time.strftime('%Y-%m-%d %H:%M:%S:  '))  # ISO-8601
-        for line in msg.splitlines():
+        for line in msg.split('\n'):
             cursor.insertText(prefix + line + '\n')
         cursor.movePosition(cursor.End)
         text.setTextCursor(cursor)
