@@ -120,6 +120,7 @@ class MainView(standard.MainWindow):
         self.branchwidget = self.branchdock.widget()
         titlebar = self.branchdock.titleBarWidget()
         titlebar.add_corner_widget(self.branchwidget.filter_button)
+        titlebar.add_corner_widget(self.branchwidget.sort_order_button)
 
         # "Submodule" widgets
         self.submodulesdock = create_dock(
@@ -250,7 +251,7 @@ class MainView(standard.MainWindow):
 
         self.save_tarball_action = add_action(
             self, N_('Save As Tarball/Zip...'),
-            partial(archive.save_archive, context, self))
+            partial(archive.save_archive, context))
 
         self.quit_action = add_action(
             self, N_('Quit'), self.close, hotkeys.QUIT)
@@ -418,6 +419,9 @@ class MainView(standard.MainWindow):
         self.lock_layout_action = add_action_bool(
             self, N_('Lock Layout'), self.set_lock_layout, False)
 
+        self.reset_layout_action = add_action(
+            self, N_('Reset Layout'), self.reset_layout)
+
         # Create the application menu
         self.menubar = QtWidgets.QMenuBar(self)
         self.setMenuBar(self.menubar)
@@ -556,6 +560,7 @@ class MainView(standard.MainWindow):
 
         # View Menu
         self.view_menu = add_menu(N_('View'), self.menubar)
+        # pylint: disable=no-member
         self.view_menu.aboutToShow.connect(
             lambda: self.build_view_menu(self.view_menu))
         self.setup_dockwidget_view_menu()
@@ -712,6 +717,7 @@ class MainView(standard.MainWindow):
 
         menu.addSeparator()
         menu.addAction(self.lock_layout_action)
+        menu.addAction(self.reset_layout_action)
 
         return menu
 
@@ -726,9 +732,13 @@ class MainView(standard.MainWindow):
         context = self.context
         menu = self.open_recent_menu
         menu.clear()
+        worktree = self.git.worktree()
         for entry in settings.recent:
-            name = entry['name']
             directory = entry['path']
+            if directory == worktree:
+                # Omit the current worktree from the "Open Recent" menu.
+                continue
+            name = entry['name']
             text = '%s %s %s' % (name, uchr(0x2192), directory)
             menu.addAction(text, cmds.run(cmd, context, directory))
 
@@ -883,6 +893,7 @@ class MainView(standard.MainWindow):
         show_status_filter = self.statuswidget.filter_widget.isVisible()
         state['show_status_filter'] = show_status_filter
         state['toolbars'] = self.toolbar_state.export_state()
+        state['ref_sort'] = self.model.ref_sort
         self.diffviewer.export_state(state)
 
         return state
@@ -899,6 +910,9 @@ class MainView(standard.MainWindow):
         toolbars = state.get('toolbars', [])
         self.toolbar_state.apply_state(toolbars)
 
+        sort_key = state.get('ref_sort', 0)
+        self.model.set_ref_sort(sort_key)
+
         diff_ok = self.diffviewer.apply_state(state)
         return base_ok and diff_ok
 
