@@ -124,6 +124,7 @@ class RepoFormWidget(FormWidget):
         self.diff_context = standard.SpinBox(value=5, mini=2, maxi=9995)
         self.merge_verbosity = standard.SpinBox(value=5, maxi=5)
         self.merge_summary = qtutils.checkbox(checked=True)
+        self.autotemplate = qtutils.checkbox(checked=False)
         self.merge_diffstat = qtutils.checkbox(checked=True)
         self.display_untracked = qtutils.checkbox(checked=True)
         self.show_path = qtutils.checkbox(checked=True)
@@ -136,6 +137,9 @@ class RepoFormWidget(FormWidget):
         tooltip = N_(
             'Prevent "Stage" from staging all files when nothing is selected')
         self.safe_mode = qtutils.checkbox(checked=False, tooltip=tooltip)
+        tooltip = N_('Enable path autocompletion in tools')
+        self.autocomplete_paths = qtutils.checkbox(
+            checked=True, tooltip=tooltip)
 
         self.add_row(N_('User Name'), self.name)
         self.add_row(N_('Email Address'), self.email)
@@ -144,13 +148,18 @@ class RepoFormWidget(FormWidget):
         self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
         self.add_row(N_('Number of Diff Context Lines'), self.diff_context)
         self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
+        self.add_row(N_('Automatically Load Commit Message Template'),
+                     self.autotemplate)
         self.add_row(N_('Show Full Paths in the Window Title'), self.show_path)
         self.add_row(N_('Show Diffstat After Merge'), self.merge_diffstat)
         self.add_row(N_('Display Untracked Files'), self.display_untracked)
         self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
         self.add_row(N_('Safe Mode'), self.safe_mode)
+        self.add_row(N_('Autocomplete Paths'), self.autocomplete_paths)
 
         self.set_config({
+            prefs.AUTOTEMPLATE:
+                (self.autotemplate, Defaults.autotemplate),
             prefs.CHECKCONFLICTS:
                 (self.check_conflicts, Defaults.check_conflicts),
             prefs.DIFFCONTEXT: (self.diff_context, Defaults.diff_context),
@@ -164,6 +173,8 @@ class RepoFormWidget(FormWidget):
             prefs.MERGE_VERBOSITY:
                 (self.merge_verbosity, Defaults.merge_verbosity),
             prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
+            prefs.AUTOCOMPLETE_PATHS: (
+                self.autocomplete_paths, Defaults.autocomplete_paths),
             prefs.SHOW_PATH: (self.show_path, Defaults.show_path),
             prefs.TABWIDTH: (self.tabwidth, Defaults.tabwidth),
             prefs.TEXTWIDTH: (self.textwidth, Defaults.textwidth),
@@ -232,6 +243,7 @@ class SettingsFormWidget(FormWidget):
             prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
         })
 
+        # pylint: disable=no-member
         self.fixed_font.currentFontChanged.connect(self.current_font_changed)
         self.font_size.valueChanged.connect(self.font_size_changed)
 
@@ -354,6 +366,7 @@ class PreferencesView(standard.Dialog):
                                         self.button_layout)
         self.setLayout(self.main_layout)
 
+        # pylint: disable=no-member
         self.tab_bar.currentChanged.connect(self.stack_widget.setCurrentIndex)
         self.stack_widget.currentChanged.connect(self.update_widget)
 
