@@ -14,25 +14,27 @@ from .diff import DiffTextEdit
 from .standard import Dialog
 
 
-def select_commits(title, revs, summaries, multiselect=True):
+def select_commits(context, title, revs, summaries, multiselect=True):
     """Use the SelectCommits to select commits from a list."""
     model = Model(revs, summaries)
     parent = qtutils.active_window()
-    dialog = SelectCommits(model, parent, title, multiselect=multiselect)
-
+    dialog = SelectCommits(
+        context, model, parent, title, multiselect=multiselect)
     return dialog.select_commits()
 
 
-def select_commits_and_output(title, revs, summaries, multiselect=True):
-    """Use the SelectCommitsAndOutput to select commits from a list and output path."""
+def select_commits_and_output(context, title, revs, summaries,
+                              multiselect=True):
+    """Select commits from a list and output path"""
     model = Model(revs, summaries)
     parent = qtutils.active_window()
-    dialog = SelectCommitsAndOutput(model, parent, title,
+    dialog = SelectCommitsAndOutput(context, model, parent, title,
                                     multiselect=multiselect)
     return dialog.select_commits_and_output()
 
 
 class Model(object):
+
     def __init__(self, revs, summaries):
         self.revisions = revs
         self.summaries = summaries
@@ -40,9 +42,10 @@ class Model(object):
 
 class SelectCommits(Dialog):
 
-    def __init__(self, model,
-                 parent=None, title=None, multiselect=True, syntax=True):
+    def __init__(self, context, model,
+                 parent=None, title=None, multiselect=True):
         Dialog.__init__(self, parent)
+        self.context = context
         self.model = model
         if title:
             self.setWindowTitle(title)
@@ -55,7 +58,7 @@ class SelectCommits(Dialog):
         commits.setSelectionMode(mode)
         commits.setAlternatingRowColors(True)
 
-        self.commit_text = DiffTextEdit(self, whitespace=False)
+        self.commit_text = DiffTextEdit(context, self, whitespace=False)
 
         self.label = QtWidgets.QLabel()
         self.label.setText(N_('Revision Expression:'))
@@ -80,6 +83,7 @@ class SelectCommits(Dialog):
                                         self.splitter, self.input_layout)
         self.setLayout(self.main_layout)
 
+        # pylint: disable=no-member
         commits.itemSelectionChanged.connect(self.commit_oid_selected)
         commits.itemDoubleClicked.connect(self.commit_oid_double_clicked)
 
@@ -112,6 +116,7 @@ class SelectCommits(Dialog):
         return self.selected_commits()
 
     def commit_oid_selected(self):
+        context = self.context
         oid = self.selected_commit()
         selected = oid is not None
         self.select_button.setEnabled(selected)
@@ -122,10 +127,10 @@ class SelectCommits(Dialog):
         self.revision.setText(oid)
         self.revision.selectAll()
         # Display the oid's commit
-        commit_diff = gitcmds.commit_diff(oid)
+        commit_diff = gitcmds.commit_diff(context, oid)
         self.commit_text.setText(commit_diff)
 
-    def commit_oid_double_clicked(self, item):
+    def commit_oid_double_clicked(self, _item):
         oid = self.selected_commit()
         if oid:
             self.accept()
@@ -133,13 +138,14 @@ class SelectCommits(Dialog):
 
 class SelectCommitsAndOutput(SelectCommits):
 
-    def __init__(self, model, parent=None, title=None, multiselect=True,
-                 syntax=True):
-        SelectCommits.__init__(self, model, parent, title, multiselect, syntax)
+    def __init__(self, context, model,
+                 parent=None, title=None, multiselect=True):
+        SelectCommits.__init__(
+            self, context, model, parent, title, multiselect)
 
         self.output_dir = 'output'
-        self.select_output = qtutils.create_button(tooltip=N_('Select output dir'),
-                                                   icon=folder())
+        self.select_output = qtutils.create_button(
+            tooltip=N_('Select output dir'), icon=folder())
         self.output_text = QtWidgets.QLineEdit()
         self.output_text.setReadOnly(True)
         self.output_text.setText(self.output_dir)
