@@ -113,6 +113,7 @@ class StartupDialog(standard.Dialog):
         qtutils.connect_button(self.new_button, self.new_repo)
         qtutils.connect_button(self.close_button, self.reject)
 
+        # pylint: disable=no-member
         self.bookmarks.activated.connect(self.open_bookmark)
 
         self.init_state(settings, self.resize_widget)
@@ -151,7 +152,7 @@ class StartupDialog(standard.Dialog):
     def clone_repo(self):
         context = self.context
         settings = self.settings
-        progress = standard.ProgressDialog('', '', self)
+        progress = standard.progress('', '', self)
         clone.clone_repo(context, self, True, settings, progress,
                          self.clone_repo_done, False)
 
