@@ -4,7 +4,6 @@
 from __future__ import absolute_import, division, unicode_literals
 import os
 import re
-from distutils.command.build import build
 from distutils.core import Command
 from distutils.dep_util import newer
 from distutils.spawn import find_executable
@@ -109,6 +108,3 @@ class build_mo(Command):
             if self.force or newer(po, mo):
                 log.info('Compile: %s -> %s' % (po, mo))
                 self.spawn(['msgfmt', '--output-file', mo, po])
-
-
-build.sub_commands.insert(0, ('build_mo', None))
