@@ -1,7 +1,7 @@
 """build_pot command for setup.py"""
-
-from __future__  import absolute_import, division, unicode_literals
-
+# pylint: disable=attribute-defined-outside-init
+# pylint: disable=no-name-in-module,import-error
+from __future__ import absolute_import, division, unicode_literals
 import os
 import glob
 from distutils import log
@@ -22,7 +22,8 @@ class build_pot(Command):
     #   - help string.
     user_options = [('build-dir=', 'd', 'Directory to put POT file'),
                     ('output=', 'o', 'POT filename'),
-                    ('lang=', None, 'Comma-separated list of languages to update po-files'),
+                    ('lang=', None,
+                     'Comma-separated list of languages to update po-files'),
                     ('no-lang', 'N', "Don't update po-files"),
                     ('english', 'E', 'Regenerate English PO file')]
     user_options = build_util.stringify_options(user_options)
@@ -43,23 +44,10 @@ class build_pot(Command):
         if self.lang is not None:
             self.lang = [i.strip() for i in self.lang.split(',') if i.strip()]
         if self.lang and self.no_lang:
-            raise DistutilsOptionError("You can't use options "
+            raise DistutilsOptionError(
+                "You can't use options "
                 "--lang=XXX and --no-lang in the same time.")
 
-    def _force_LF(self, src, dst=None):
-        f = open(src, 'rU')
-        try:
-            content = f.read()
-        finally:
-            f.close()
-        if dst is None:
-            dst = src
-        f = open(dst, 'wb')
-        try:
-            f.write(build_util.encode(content))
-        finally:
-            f.close()
-
     def run(self):
         """Run xgettext for project sources"""
         # project name based on `name` argument in setup() call
@@ -73,7 +61,8 @@ class build_pot(Command):
         if not os.path.isdir(self.build_dir):
             log.info('Make directory: ' + self.build_dir)
             os.makedirs(self.build_dir)
-        self.spawn([
+
+        cmd = [
             'xgettext',
             '--language=Python',
             '--keyword=N_',
@@ -82,12 +71,15 @@ class build_pot(Command):
             '--omit-header',
             '--sort-output',
             '--output-dir', self.build_dir,
-            '--output', self.output] +
-            glob.glob('bin/git-*') +
-            glob.glob('share/git-cola/bin/git-*') +
-            glob.glob('cola/*.py') +
-            glob.glob('cola/*/*.py'))
-        self._force_LF(fullname)
+            '--output', self.output,
+        ]
+        cmd.extend(glob.glob('bin/git-*'))
+        cmd.extend(glob.glob('share/git-cola/bin/git-*'))
+        cmd.extend(glob.glob('cola/*.py'))
+        cmd.extend(glob.glob('cola/*/*.py'))
+        self.spawn(cmd)
+
+        _force_LF(fullname)
         # regenerate english PO
         if self.english:
             log.info('Regenerating English PO file...')
@@ -104,7 +96,7 @@ class build_pot(Command):
         # search and update all po-files
         if self.no_lang:
             return
-        for po in glob.glob(os.path.join(self.build_dir,'*.po')):
+        for po in glob.glob(os.path.join(self.build_dir, '*.po')):
             if self.lang is not None:
                 po_lang = os.path.splitext(os.path.basename(po))[0]
                 if prj_name and po_lang.startswith(prj_name+'-'):
@@ -121,5 +113,20 @@ class build_pot(Command):
                 po, fullname])
             # force LF line-endings
             log.info('%s --> %s' % (new_po, po))
-            self._force_LF(new_po, po)
+            _force_LF(new_po, po)
             os.unlink(new_po)
+
+
+def _force_LF(src, dst=None):
+    f = open(src, 'rU')
+    try:
+        content = f.read()
+    finally:
+        f.close()
+    if dst is None:
+        dst = src
+    f = open(dst, 'wb')
+    try:
+        f.write(build_util.encode(content))
+    finally:
+        f.close()
